@@ -78,9 +78,23 @@ GST_DEBUG_CATEGORY_EXTERN (alsa_debug);
 
 #define GST_ALSA_MIN_RATE	8000
 #define GST_ALSA_MAX_RATE	192000
+#define GST_ALSA_MIN_PERIOD_CNT	2
+#define GST_ALSA_MAX_PERIOD_CNT	64
+#define GST_ALSA_MIN_PERIOD_SZ	2
+#define GST_ALSA_MAX_PERIOD_SZ	8192
+#define GST_ALSA_MIN_BUFFER_SZ	GST_ALSA_MIN_PERIOD_CNT*GST_ALSA_MIN_PERIOD_SZ
+#define GST_ALSA_MAX_BUFFER_SZ	65536
 #define GST_ALSA_MAX_TRACKS	64 /* we don't support more than 64 tracks */
 #define GST_ALSA_MAX_CHANNELS	32 /* tracks can have up to 32 channels */
 
+/* a few alsa functions return an int value and a 'direction', -1, 0 or +1
+  0 = exact value
+ -1 = real value is up to 1 before given value
+ +1 = real value is up to 1 after given value
+*/
+#define GST_ALSA_DIR_MIN(i) ((i == 1) ? 1 : 0)
+#define GST_ALSA_DIR_MAX(i) ((i ==-1) ?-1 : 0)
+
 /* Mono is 1 channel ; the 5.1 standard is 6 channels. The value for
    GST_ALSA_MAX_CHANNELS comes from alsa/mixer.h. */
 
@@ -147,6 +161,7 @@ struct _GstAlsa {
   GstAlsaFormat *		format;		/* NULL if undefined */
   gboolean			mmap; 		/* use mmap transmit (fast) or read/write (sloooow) */
   GstAlsaTransmitFunction	transmit;
+  GstCaps *			cached_caps;	/* we cache caps to speed up get_caps */
 
   /* latency / performance parameters */
   snd_pcm_uframes_t		period_size;
@@ -156,6 +171,7 @@ struct _GstAlsa {
 
   /* clocking */
   GstAlsaClock *		clock;		/* our provided clock */
+  GstClock *			ext_clock;	/* externally set clock. */
   snd_pcm_uframes_t		played; 	/* samples transmitted since last sync 
 						   This thing actually is our master clock.
 						   We will event insert silent samples or
