@@ -132,6 +132,20 @@ gst_alsa_mixer_init (GstAlsaMixer * mixer)
   mixer->mixer_handle = NULL;
 }
 
+static int
+cb_alsa_call (snd_mixer_t * m, unsigned int mask, snd_mixer_elem_t * selem)
+{
+  GstAlsaMixer *mixer = snd_mixer_get_callback_private (m);
+  const gchar *name = NULL;
+
+  if (selem)
+    name = snd_mixer_selem_get_name (selem);
+
+  GST_LOG_OBJECT (mixer, "Mixer event 0x%x on element %s", mask, name);
+
+  return 0;
+}
+
 static gboolean
 gst_alsa_mixer_open (GstAlsaMixer * mixer)
 {
@@ -148,6 +162,8 @@ gst_alsa_mixer_open (GstAlsaMixer * mixer)
     mixer->mixer_handle = NULL;
     return FALSE;
   }
+  snd_mixer_set_callback (mixer->mixer_handle, cb_alsa_call);
+  snd_mixer_set_callback_private (mixer->mixer_handle, mixer);
 
   if (!strncmp (alsa->device, "hw:", 3))
     nocomma = g_strdup (alsa->device);
@@ -240,6 +256,29 @@ gst_alsa_mixer_supported (GstImplementsInterface * iface, GType iface_type)
   return (GST_ALSA_MIXER (iface)->mixer_handle != NULL);
 }
 
+static int
+cb_alsa_elem (snd_mixer_elem_t * selem, unsigned int mask)
+{
+  GstMixer *mixer = snd_mixer_elem_get_callback_private (selem);
+  const gchar *name = snd_mixer_selem_get_name (selem);
+
+  GST_LOG_OBJECT (mixer, "Element event 0x%x on element %s", mask, name);
+
+  if (mask == SND_CTL_EVENT_MASK_REMOVE) {
+    GstAlsaMixer *alsa_mixer = GST_ALSA_MIXER (mixer);
+    const GList *item;
+
+    for (item = alsa_mixer->tracklist; item; item = item->next) {
+      GstAlsaMixerTrack *alsa_track = item->data;
+
+      if (alsa_track->element == selem)
+        alsa_track->disabled = TRUE;
+    }
+  }
+
+  return 0;
+}
+
 static void
 gst_alsa_mixer_build_list (GstAlsaMixer * mixer)
 {
@@ -264,28 +303,45 @@ gst_alsa_mixer_build_list (GstAlsaMixer * mixer)
 
   /* build track list */
   for (i = 0; i < count; i++) {
-    gint channels = 0;
+    GList *item;
+    gint channels = 0, samename = 0;
     gint flags = GST_MIXER_TRACK_OUTPUT;
+    gboolean got_it = FALSE;
 
     if (snd_mixer_selem_has_capture_switch (element)) {
       if (dir != GST_PAD_SRC && dir != GST_PAD_UNKNOWN)
-        continue;
+        goto next;
       flags = GST_MIXER_TRACK_INPUT;
     } else {
       if (dir != GST_PAD_SINK && dir != GST_PAD_UNKNOWN)
-        continue;
+        goto next;
+    }
+
+    /* prevent dup names */
+    for (item = mixer->tracklist; item != NULL; item = item->next) {
+      snd_mixer_elem_t *temp;
+
+      if (GST_IS_ALSA_MIXER_OPTIONS (item->data))
+        temp = GST_ALSA_MIXER_OPTIONS (item->data)->element;
+      else
+        temp = GST_ALSA_MIXER_TRACK (item->data)->element;
+
+      if (!strcmp (snd_mixer_selem_get_name (element),
+              snd_mixer_selem_get_name (temp)))
+        samename++;
     }
 
     if (snd_mixer_selem_has_capture_volume (element)) {
       while (snd_mixer_selem_has_capture_channel (element, channels))
         channels++;
-      if (first) {
-        first = FALSE;
-        flags |= GST_MIXER_TRACK_MASTER;
-      }
-      track = gst_alsa_mixer_track_new (element, i, channels,
-          flags, GST_ALSA_MIXER_TRACK_CAPTURE);
+      track = gst_alsa_mixer_track_new (element, samename,
+          i, channels, flags, GST_ALSA_MIXER_TRACK_CAPTURE);
       mixer->tracklist = g_list_append (mixer->tracklist, track);
+      got_it = TRUE;
+
+      /* there might be another volume slider; make that playback */
+      flags &= ~GST_MIXER_TRACK_INPUT;
+      flags |= GST_MIXER_TRACK_OUTPUT;
     }
 
     if (snd_mixer_selem_has_playback_volume (element)) {
@@ -295,16 +351,32 @@ gst_alsa_mixer_build_list (GstAlsaMixer * mixer)
         first = FALSE;
         flags |= GST_MIXER_TRACK_MASTER;
       }
-      track = gst_alsa_mixer_track_new (element, i, channels,
-          flags, GST_ALSA_MIXER_TRACK_PLAYBACK);
+      track = gst_alsa_mixer_track_new (element, samename,
+          i, channels, flags, GST_ALSA_MIXER_TRACK_PLAYBACK);
       mixer->tracklist = g_list_append (mixer->tracklist, track);
+      got_it = TRUE;
     }
 
     if (snd_mixer_selem_is_enumerated (element)) {
       opts = gst_alsa_mixer_options_new (element, i);
       mixer->tracklist = g_list_append (mixer->tracklist, opts);
+      got_it = TRUE;
     }
 
+    if (!got_it) {
+      if (flags == GST_MIXER_TRACK_OUTPUT &&
+          snd_mixer_selem_has_playback_switch (element)) {
+        /* simple mute switch */
+        track = gst_alsa_mixer_track_new (element, samename,
+            i, 0, flags, GST_ALSA_MIXER_TRACK_PLAYBACK);
+        mixer->tracklist = g_list_append (mixer->tracklist, track);
+      }
+    }
+
+    snd_mixer_elem_set_callback (element, cb_alsa_elem);
+    snd_mixer_elem_set_callback_private (element, mixer);
+
+  next:
     element = snd_mixer_elem_next (element);
   }
 }
@@ -363,6 +435,35 @@ gst_alsa_mixer_list_tracks (GstMixer * mixer)
   return (const GList *) alsa_mixer->tracklist;
 }
 
+static void
+gst_alsa_mixer_update (GstAlsaMixer * alsa_mixer,
+    GstAlsaMixerTrack * alsa_track)
+{
+  GstMixerTrack *track;
+  int v = 0;
+
+  snd_mixer_handle_events (alsa_mixer->mixer_handle);
+  if (!alsa_track || alsa_track->disabled)
+    return;
+  track = (GstMixerTrack *) alsa_track;
+
+  /* Any updates in flags? */
+  if (snd_mixer_selem_has_playback_switch (alsa_track->element)) {
+    snd_mixer_selem_get_playback_switch (alsa_track->element, 0, &v);
+    if (v)
+      track->flags &= ~GST_MIXER_TRACK_MUTE;
+    else
+      track->flags |= GST_MIXER_TRACK_MUTE;
+  }
+  if (alsa_track->alsa_flags & GST_ALSA_MIXER_TRACK_CAPTURE) {
+    snd_mixer_selem_get_capture_switch (alsa_track->element, 0, &v);
+    if (!v)
+      track->flags &= ~GST_MIXER_TRACK_RECORD;
+    else
+      track->flags |= GST_MIXER_TRACK_RECORD;
+  }
+}
+
 static void
 gst_alsa_mixer_get_volume (GstMixer * mixer,
     GstMixerTrack * track, gint * volumes)
@@ -372,8 +473,11 @@ gst_alsa_mixer_get_volume (GstMixer * mixer,
 
   g_return_if_fail (GST_ALSA_MIXER (mixer)->mixer_handle != NULL);
 
-  if (track->flags & GST_MIXER_TRACK_MUTE &&
-      !snd_mixer_selem_has_playback_switch (alsa_track->element)) {
+  gst_alsa_mixer_update (GST_ALSA_MIXER (mixer), alsa_track);
+
+  if (alsa_track->disabled ||
+      (track->flags & GST_MIXER_TRACK_MUTE &&
+       !snd_mixer_selem_has_playback_switch (alsa_track->element))) {
     for (i = 0; i < track->num_channels; i++)
       volumes[i] = alsa_track->volumes[i];
   } else {
@@ -385,7 +489,9 @@ gst_alsa_mixer_get_volume (GstMixer * mixer,
       } else if (alsa_track->alsa_flags & GST_ALSA_MIXER_TRACK_CAPTURE) {
         snd_mixer_selem_get_capture_volume (alsa_track->element, i, &tmp);
       }
-      volumes[i] = (gint) tmp;
+      if (!(track->flags & GST_MIXER_TRACK_MUTE)) {
+        volumes[i] = (gint) tmp;
+      }
     }
   }
 }
@@ -399,6 +505,10 @@ gst_alsa_mixer_set_volume (GstMixer * mixer,
 
   g_return_if_fail (GST_ALSA_MIXER (mixer)->mixer_handle != NULL);
 
+  gst_alsa_mixer_update (GST_ALSA_MIXER (mixer), alsa_track);
+  if (alsa_track->disabled)
+    return;
+
   /* only set the volume with ALSA lib if the track isn't muted. */
   for (i = 0; i < track->num_channels; i++) {
     alsa_track->volumes[i] = volumes[i];
@@ -424,6 +534,10 @@ gst_alsa_mixer_set_mute (GstMixer * mixer, GstMixerTrack * track, gboolean mute)
 
   g_return_if_fail (GST_ALSA_MIXER (mixer)->mixer_handle != NULL);
 
+  gst_alsa_mixer_update (GST_ALSA_MIXER (mixer), alsa_track);
+  if (alsa_track->disabled)
+    return;
+
   if (mute) {
     track->flags |= GST_MIXER_TRACK_MUTE;
   } else {
@@ -453,6 +567,10 @@ gst_alsa_mixer_set_record (GstMixer * mixer,
 
   g_return_if_fail (GST_ALSA_MIXER (mixer)->mixer_handle != NULL);
 
+  gst_alsa_mixer_update (GST_ALSA_MIXER (mixer), alsa_track);
+  if (alsa_track->disabled)
+    return;
+
   if (record) {
     track->flags |= GST_MIXER_TRACK_RECORD;
   } else {
@@ -472,6 +590,8 @@ gst_alsa_mixer_set_option (GstMixer * mixer,
 
   g_return_if_fail (GST_ALSA_MIXER (mixer)->mixer_handle != NULL);
 
+  gst_alsa_mixer_update (GST_ALSA_MIXER (mixer), NULL);
+
   for (item = opts->values; item != NULL; item = item->next, n++) {
     if (!strcmp (item->data, value)) {
       idx = n;
@@ -492,6 +612,8 @@ gst_alsa_mixer_get_option (GstMixer * mixer, GstMixerOptions * opts)
 
   g_return_val_if_fail (GST_ALSA_MIXER (mixer)->mixer_handle != NULL, NULL);
 
+  gst_alsa_mixer_update (GST_ALSA_MIXER (mixer), NULL);
+
   snd_mixer_selem_get_enum_item (alsa_opts->element, 0, &idx);
 
   return g_list_nth_data (opts->values, idx);
