@@ -730,7 +730,7 @@ cdparanoia_open (CDParanoia * src)
   }
 
   /* save thse ones - skip lead-in */
-  src->first_sector = cdda_track_firstsector (src->d, 1);
+  src->first_sector = 0; //cdda_track_firstsector (src->d, 1);
   src->last_sector = cdda_disc_lastsector (src->d);
 
   /* this is the default segment we will play */
@@ -1055,10 +1055,11 @@ cdparanoia_convert (GstPad * pad,
             *dest_value = sector;
           } else if (*dest_format == track_format) {
             /* if we go past the last sector, make sure to report the last track */
-            if (sector > src->last_sector)
+            if (sector > src->last_sector - src->first_sector)
               *dest_value = cdda_sector_gettrack (src->d, src->last_sector);
             else
-              *dest_value = cdda_sector_gettrack (src->d, sector) - 1;
+              *dest_value = cdda_sector_gettrack (src->d,
+                  sector + src->first_sector) - 1;
           } else {
             return FALSE;
           }
