@@ -21,20 +21,68 @@
 #include "config.h"
 #endif
 #include <string.h>
+#include <math.h>
 
 /* First, include the header file for the plugin, to bring in the
  * object definition and other useful things.
  */
 #include "gstdvdec.h"
 
+/* DV output has two modes, normal and wide. The resolution is the same in both
+ * cases: 720 pixels wide by 576 pixels tall in PAL format, and 720x480 for
+ * NTSC.
+ *
+ * Each of the modes has its own pixel aspect ratio, which is fixed in practice
+ * by ITU-R BT.601 (also known as "CCIR-601" or "Rec.601"). Or so claims a
+ * reference that I culled from the reliable "internet",
+ * http://www.mir.com/DMG/aspect.html. Normal PAL is 59/54 and normal NTSC is
+ * 10/11. Because the pixel resolution is the same for both cases, we can get
+ * the pixel aspect ratio for wide recordings by multiplying by the ratio of
+ * display aspect ratios, 16/9 (for wide) divided by 4/3 (for normal):
+ *
+ * Wide NTSC: 10/11 * (16/9)/(4/3) = 40/33
+ * Wide PAL: 59/54 * (16/9)/(4/3) = 118/81
+ *
+ * However, the pixel resolution coming out of a DV source does not combine with
+ * the standard pixel aspect ratios to give a proper display aspect ratio. An
+ * image 480 pixels tall, with a 4:3 display aspect ratio, will be 768 pixels
+ * wide. But, if we take the normal PAL aspect ratio of 59/54, and multiply it
+ * with the width of the DV image (720 pixels), we get 786.666..., which is
+ * nonintegral and too wide. The camera is not outputting a 4:3 image.
+ * 
+ * If the video sink for this stream has fixed dimensions (such as for
+ * fullscreen playback, or for a java applet in a web page), you then have two
+ * choices. Either you show the whole image, but pad the image with black
+ * borders on the top and bottom (like watching a widescreen video on a 4:3
+ * device), or you crop the video to the proper ratio. Apparently the latter is
+ * the standard practice.
+ *
+ * For its part, GStreamer is concerned with accuracy and preservation of
+ * information. This element outputs the 720x576 or 720x480 video that it
+ * recieves, noting the proper aspect ratio. This should not be a problem for
+ * windowed applications, which can change size to fit the video. Applications
+ * with fixed size requirements should decide whether to crop or pad. If you
+ * decide to crop, the videocrop element might be useful to you.
+ */
+
 #define NTSC_HEIGHT 480
 #define NTSC_BUFFER 120000
-#define NTSC_FRAMERATE 29.997
+#define NTSC_FRAMERATE 30000/1001.
 
 #define PAL_HEIGHT 576
 #define PAL_BUFFER 144000
 #define PAL_FRAMERATE 25.0
 
+#define PAL_NORMAL_PAR_X	59
+#define PAL_NORMAL_PAR_Y	54
+#define PAL_WIDE_PAR_X		118
+#define PAL_WIDE_PAR_Y		81
+
+#define NTSC_NORMAL_PAR_X	10
+#define NTSC_NORMAL_PAR_Y	11
+#define NTSC_WIDE_PAR_X		40
+#define NTSC_WIDE_PAR_Y		33
+
 /* The ElementDetails structure gives a human-readable description
  * of the plugin, as well as author and version data.
  */
@@ -98,21 +146,33 @@ static GstStaticPadTemplate video_src_temp = GST_STATIC_PAD_TEMPLATE ("video",
         "width = (int) 720, "
         "height = (int) { "
         G_STRINGIFY (NTSC_HEIGHT) ", " G_STRINGIFY (PAL_HEIGHT)
-        " }, " "framerate = (double) [ 1.0, 60.0 ];"
+        " }, "
+        "pixel-aspect-ratio=(fraction) { "
+        G_STRINGIFY (PAL_NORMAL_PAR_X) "/" G_STRINGIFY (PAL_NORMAL_PAR_Y) ","
+        G_STRINGIFY (PAL_WIDE_PAR_X) "/" G_STRINGIFY (PAL_WIDE_PAR_Y) ","
+        G_STRINGIFY (NTSC_NORMAL_PAR_X) "/" G_STRINGIFY (NTSC_NORMAL_PAR_Y) ","
+        G_STRINGIFY (NTSC_WIDE_PAR_X) "/" G_STRINGIFY (NTSC_WIDE_PAR_Y) "},"
+        "framerate = (double) [ 1.0, 60.0 ];"
         //"framerate = (double) { "
         //G_STRINGIFY (PAL_FRAMERATE) ", " G_STRINGIFY (NTSC_FRAMERATE)
         //" }; "
         "video/x-raw-rgb, "
         "bpp = (int) 32, "
-        "depth = (int) 32, "
+        "depth = (int) 24, "
         "endianness = (int) " G_STRINGIFY (G_BIG_ENDIAN) ", "
-        "red_mask =   (int) 0x00ff0000, "
-        "green_mask = (int) 0x0000ff00, "
-        "blue_mask =  (int) 0x000000ff, "
+        "red_mask =   (int) 0x0000ff00, "
+        "green_mask = (int) 0x00ff0000, "
+        "blue_mask =  (int) 0xff000000, "
         "width = (int) 720, "
         "height = (int) { "
         G_STRINGIFY (NTSC_HEIGHT) ", " G_STRINGIFY (PAL_HEIGHT)
-        " }, " "framerate = (double) [ 1.0, 60.0 ];"
+        " }, "
+        "pixel-aspect-ratio=(fraction) { "
+        G_STRINGIFY (PAL_NORMAL_PAR_X) "/" G_STRINGIFY (PAL_NORMAL_PAR_Y) ","
+        G_STRINGIFY (PAL_WIDE_PAR_X) "/" G_STRINGIFY (PAL_WIDE_PAR_Y) ","
+        G_STRINGIFY (NTSC_NORMAL_PAR_X) "/" G_STRINGIFY (NTSC_NORMAL_PAR_Y) ","
+        G_STRINGIFY (NTSC_WIDE_PAR_X) "/" G_STRINGIFY (NTSC_WIDE_PAR_Y) "},"
+        "framerate = (double) [ 1.0, 60.0 ];"
         //"framerate = (double) { "
         //G_STRINGIFY (PAL_FRAMERATE) ", " G_STRINGIFY (NTSC_FRAMERATE)
         //" }; "
@@ -126,7 +186,13 @@ static GstStaticPadTemplate video_src_temp = GST_STATIC_PAD_TEMPLATE ("video",
         "width = (int) 720, "
         "height = (int) { "
         G_STRINGIFY (NTSC_HEIGHT) ", " G_STRINGIFY (PAL_HEIGHT)
-        " }, " "framerate = (double) [ 1.0, 60.0 ]"
+        " }, "
+        "pixel-aspect-ratio=(fraction) { "
+        G_STRINGIFY (PAL_NORMAL_PAR_X) "/" G_STRINGIFY (PAL_NORMAL_PAR_Y) ","
+        G_STRINGIFY (PAL_WIDE_PAR_X) "/" G_STRINGIFY (PAL_WIDE_PAR_Y) ","
+        G_STRINGIFY (NTSC_NORMAL_PAR_X) "/" G_STRINGIFY (NTSC_NORMAL_PAR_Y) ","
+        G_STRINGIFY (NTSC_WIDE_PAR_X) "/" G_STRINGIFY (NTSC_WIDE_PAR_Y) "},"
+        "framerate = (double) [ 1.0, 60.0 ]"
         //"framerate = (double) { "
         //G_STRINGIFY (PAL_FRAMERATE) ", " G_STRINGIFY (NTSC_FRAMERATE)
         //" }"
@@ -140,9 +206,9 @@ static GstStaticPadTemplate audio_src_temp = GST_STATIC_PAD_TEMPLATE ("audio",
         "depth = (int) 16, "
         "width = (int) 16, "
         "signed = (boolean) TRUE, "
-        "channels = (int) 2, "
+        "channels = (int) {2, 4}, "
         "endianness = (int) " G_STRINGIFY (G_LITTLE_ENDIAN) ", "
-        "rate = (int) [ 4000, 48000 ]")
+        "rate = (int) { 32000, 44100, 48000 }")
     );
 
 #define GST_TYPE_DVDEC_QUALITY (gst_dvdec_quality_get_type())
@@ -372,6 +438,7 @@ gst_dvdec_init (GstDVDec * dvdec)
   dvdec->height = 0;
   dvdec->frequency = 0;
   dvdec->channels = 0;
+  dvdec->wide = FALSE;
   dvdec->drop_factor = 1;
 
   dvdec->clamp_luma = FALSE;
@@ -627,6 +694,9 @@ gst_dvdec_handle_sink_event (GstDVDec * dvdec)
         gst_event_ref (event);
         gst_pad_push (dvdec->audiosrcpad, GST_DATA (event));
       }
+      if (type == GST_EVENT_EOS) {
+        gst_element_set_eos (GST_ELEMENT (dvdec));
+      }
       break;
     }
     case GST_EVENT_DISCONTINUOUS:
@@ -740,7 +810,25 @@ gst_dvdec_video_getcaps (GstPad * pad)
 
   if (dvdec->found_header) {
     int i;
+    gint par_x, par_y;
 
+    if (dvdec->PAL) {
+      if (dvdec->wide) {
+        par_x = PAL_WIDE_PAR_X;
+        par_y = PAL_WIDE_PAR_Y;
+      } else {
+        par_x = PAL_NORMAL_PAR_X;
+        par_y = PAL_NORMAL_PAR_Y;
+      }
+    } else {
+      if (dvdec->wide) {
+        par_x = NTSC_WIDE_PAR_X;
+        par_y = NTSC_WIDE_PAR_Y;
+      } else {
+        par_x = NTSC_NORMAL_PAR_X;
+        par_y = NTSC_NORMAL_PAR_Y;
+      }
+    }
     /* set the height */
     for (i = 0; i < gst_caps_get_size (caps); i++) {
       GstStructure *structure = gst_caps_get_structure (caps, i);
@@ -748,7 +836,7 @@ gst_dvdec_video_getcaps (GstPad * pad)
       gst_structure_set (structure,
           "height", G_TYPE_INT, dvdec->height,
           "framerate", G_TYPE_DOUBLE, dvdec->framerate / dvdec->drop_factor,
-          NULL);
+          "pixel-aspect-ratio", GST_TYPE_FRACTION, par_x, par_y, NULL);
     }
   }
 
@@ -777,8 +865,9 @@ gst_dvdec_video_link (GstPad * pad, const GstCaps * caps)
       !gst_structure_get_double (structure, "framerate", &framerate))
     return GST_PAD_LINK_REFUSED;
 
-  if ((height != dvdec->height)
-      || (framerate != dvdec->framerate / dvdec->drop_factor))
+  /* allow a margin of error for the framerate caused by float rounding errors */
+  if ((height != dvdec->height) ||
+      (fabs (framerate - (dvdec->framerate / dvdec->drop_factor)) > 0.00000001))
     return GST_PAD_LINK_REFUSED;
 
   if (strcmp (gst_structure_get_name (structure), "video/x-raw-rgb") == 0) {
@@ -836,6 +925,7 @@ gst_dvdec_loop (GstElement * element)
   guint32 length, got_bytes;
   GstClockTime ts, duration;
   gdouble fps;
+  gboolean wide;
 
   dvdec = GST_DVDEC (element);
 
@@ -866,6 +956,7 @@ gst_dvdec_loop (GstElement * element)
   fps = (dvdec->PAL ? PAL_FRAMERATE : NTSC_FRAMERATE);
   height = (dvdec->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
   length = (dvdec->PAL ? PAL_BUFFER : NTSC_BUFFER);
+  wide = dv_format_wide (dvdec->decoder);
 
   if (length != dvdec->length) {
     dvdec->length = length;
@@ -956,9 +1047,11 @@ gst_dvdec_loop (GstElement * element)
     }
     dvdec->framecount = 0;
 
-    if ((dvdec->framerate != fps) || (dvdec->height != height)) {
+    if ((dvdec->framerate != fps) || (dvdec->height != height)
+        || dvdec->wide != wide) {
       dvdec->height = height;
       dvdec->framerate = fps;
+      dvdec->wide = wide;
 
       if (GST_PAD_LINK_FAILED (gst_pad_renegotiate (dvdec->videosrcpad))) {
         GST_ELEMENT_ERROR (dvdec, CORE, NEGOTIATION, (NULL), (NULL));
@@ -992,6 +1085,7 @@ gst_dvdec_loop (GstElement * element)
   } else {
     dvdec->height = height;
     dvdec->framerate = fps;
+    dvdec->wide = wide;
   }
 
 end:
