@@ -60,34 +60,34 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
 #define STATIC_CAPS \
   STATIC_INT_CAPS (16)
 #if 0
-"; "
-STATIC_INT_CAPS (24)
-    "; "
-STATIC_INT_CAPS (32)
-    "; "
-STATIC_FLOAT_CAPS (32)
-    "; "
+#define NOTUSED "; " \
+STATIC_INT_CAPS (24) \
+    "; " \
+STATIC_INT_CAPS (32) \
+    "; " \
+STATIC_FLOAT_CAPS (32) \
+    "; " \
 STATIC_FLOAT_CAPS (64)
 #endif
-     static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
+static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (STATIC_CAPS)
     );
 
-     static void gst_faad_base_init (GstFaadClass * klass);
-     static void gst_faad_class_init (GstFaadClass * klass);
-     static void gst_faad_init (GstFaad * faad);
+static void gst_faad_base_init (GstFaadClass * klass);
+static void gst_faad_class_init (GstFaadClass * klass);
+static void gst_faad_init (GstFaad * faad);
 
-     static GstPadLinkReturn
-         gst_faad_sinkconnect (GstPad * pad, const GstCaps * caps);
-     static GstPadLinkReturn
-         gst_faad_srcconnect (GstPad * pad, const GstCaps * caps);
-     static GstCaps *gst_faad_srcgetcaps (GstPad * pad);
-     static void gst_faad_chain (GstPad * pad, GstData * data);
-     static GstElementStateReturn gst_faad_change_state (GstElement * element);
+static GstPadLinkReturn
+gst_faad_sinkconnect (GstPad * pad, const GstCaps * caps);
+static GstPadLinkReturn
+gst_faad_srcconnect (GstPad * pad, const GstCaps * caps);
+static GstCaps *gst_faad_srcgetcaps (GstPad * pad);
+static void gst_faad_chain (GstPad * pad, GstData * data);
+static GstElementStateReturn gst_faad_change_state (GstElement * element);
 
-     static GstElementClass *parent_class = NULL;
+static GstElementClass *parent_class = NULL;
 
 /* static guint gst_faad_signals[LAST_SIGNAL] = { 0 }; */
 
@@ -282,11 +282,17 @@ gst_faad_sinkconnect (GstPad * pad, const GstCaps * caps)
   const GValue *value;
   GstBuffer *buf;
 
+  /* Assume raw stream */
+  faad->packetised = FALSE;
+
   if ((value = gst_structure_get_value (str, "codec_data"))) {
     gulong samplerate;
     guchar channels;
 
+    /* We have codec data, means packetised stream */
+    faad->packetised = TRUE;
     buf = g_value_get_boxed (value);
+
     /* someone forgot that char can be unsigned when writing the API */
     if ((gint8) faacDecInit2 (faad->handle, GST_BUFFER_DATA (buf),
             GST_BUFFER_SIZE (buf), &samplerate, &channels) < 0)
@@ -316,18 +322,18 @@ gst_faad_srcgetcaps (GstPad * pad)
 {
   GstFaad *faad = GST_FAAD (gst_pad_get_parent (pad));
   static GstAudioChannelPosition *supported_positions = NULL;
-  static gint num_supported_positions = LFE_CHANNEL - FRONT_CHANNEL_CENTER;
+  static gint num_supported_positions = LFE_CHANNEL - FRONT_CHANNEL_CENTER + 1;
   GstCaps *templ;
 
   if (!supported_positions) {
-    guchar *supported_fpos = g_new0 (guchar,
-        LFE_CHANNEL - FRONT_CHANNEL_CENTER);
+    guchar *supported_fpos = g_new0 (guchar, num_supported_positions);
     gint n;
 
-    for (n = 0; n < LFE_CHANNEL - FRONT_CHANNEL_CENTER; n++) {
+    for (n = 0; n < num_supported_positions; n++) {
       supported_fpos[n] = n + FRONT_CHANNEL_CENTER;
     }
-    supported_positions = gst_faad_chanpos_to_gst (supported_fpos, n);
+    supported_positions = gst_faad_chanpos_to_gst (supported_fpos,
+        num_supported_positions);
     g_free (supported_fpos);
   }
 
@@ -540,12 +546,14 @@ static void
 gst_faad_chain (GstPad * pad, GstData * data)
 {
   guint input_size;
+  guint skip_bytes = 0;
   guchar *input_data;
   GstFaad *faad = GST_FAAD (gst_pad_get_parent (pad));
   GstBuffer *buf, *outbuf;
   faacDecFrameInfo *info;
   guint64 next_ts;
   void *out;
+  gboolean run_loop = TRUE;
 
   if (GST_IS_EVENT (data)) {
     GstEvent *event = GST_EVENT (data);
@@ -579,12 +587,20 @@ gst_faad_chain (GstPad * pad, GstData * data)
   if (!faad->init) {
     gulong samplerate;
     guchar channels;
+    glong init_res;
 
-    faacDecInit (faad->handle,
+    init_res = faacDecInit (faad->handle,
         GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf), &samplerate, &channels);
+    if (init_res < 0) {
+      GST_ELEMENT_ERROR (faad, STREAM, DECODE, (NULL),
+          ("Failed to init decoder from stream"));
+      return;
+    }
+    skip_bytes = init_res;
     faad->init = TRUE;
 
     /* store for renegotiation later on */
+    /* FIXME: that's moot, info will get zeroed in DecDecode() */
     info->samplerate = samplerate;
     info->channels = channels;
   } else {
@@ -595,9 +611,26 @@ gst_faad_chain (GstPad * pad, GstData * data)
   /* decode cycle */
   input_data = GST_BUFFER_DATA (buf);
   input_size = GST_BUFFER_SIZE (buf);
-  info->bytesconsumed = input_size;
-  while (input_size >= FAAD_MIN_STREAMSIZE && info->bytesconsumed > 0) {
-    out = faacDecDecode (faad->handle, info, input_data, input_size);
+  info->bytesconsumed = input_size - skip_bytes;
+
+  if (!faad->packetised) {
+    /* We must check that ourselves for raw stream */
+    run_loop = (input_size >= FAAD_MIN_STREAMSIZE);
+  }
+
+  while ((input_size > 0) && run_loop) {
+
+    if (faad->packetised) {
+      /* Only one packet per buffer, no matter how much is really consumed */
+      run_loop = FALSE;
+    } else {
+      if (input_size < FAAD_MIN_STREAMSIZE || info->bytesconsumed <= 0) {
+        break;
+      }
+    }
+
+    out = faacDecDecode (faad->handle, info, input_data + skip_bytes,
+        input_size - skip_bytes);
     if (info->error) {
       GST_ELEMENT_ERROR (faad, STREAM, DECODE, (NULL),
           ("Failed to decode buffer: %s",
@@ -654,14 +687,16 @@ gst_faad_chain (GstPad * pad, GstData * data)
         GST_BUFFER_TIMESTAMP (outbuf) = next_ts;
         GST_BUFFER_DURATION (outbuf) =
             (guint64) GST_SECOND *info->samples / faad->samplerate;
-        next_ts += GST_BUFFER_DURATION (outbuf);
+        if (GST_CLOCK_TIME_IS_VALID (next_ts)) {
+          next_ts += GST_BUFFER_DURATION (outbuf);
+        }
         gst_pad_push (faad->srcpad, GST_DATA (outbuf));
       }
     }
   }
 
-  /* Keep the leftovers */
-  if (input_size > 0) {
+  /* Keep the leftovers in raw stream */
+  if (input_size > 0 && !faad->packetised) {
     if (input_size < GST_BUFFER_SIZE (buf)) {
       faad->tempbuf = gst_buffer_create_sub (buf,
           GST_BUFFER_SIZE (buf) - input_size, input_size);
@@ -690,7 +725,7 @@ gst_faad_change_state (GstElement * element)
 
         conf = faacDecGetCurrentConfiguration (faad->handle);
         conf->defObjectType = LC;
-        conf->dontUpSampleImplicitSBR = 1;
+        //conf->dontUpSampleImplicitSBR = 1;
         faacDecSetConfiguration (faad->handle, conf);
       }
       break;
