@@ -88,8 +88,7 @@ src_templ (void)
     GstCaps *caps;
     GstStructure *structure;
     GValue list = { 0 }
-    , fps =
-    {
+    , fps = {
     0}
     , fmt = {
     0};
@@ -150,6 +149,7 @@ static void gst_mpeg2dec_class_init (GstMpeg2decClass * klass);
 static void gst_mpeg2dec_init (GstMpeg2dec * mpeg2dec);
 
 static void gst_mpeg2dec_dispose (GObject * object);
+static void gst_mpeg2dec_reset (GstMpeg2dec * mpeg2dec);
 
 static void gst_mpeg2dec_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -177,6 +177,11 @@ static void gst_mpeg2dec_chain (GstPad * pad, GstData * _data);
 
 static GstElementClass *parent_class = NULL;
 
+static GstBuffer *crop_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * input);
+static gboolean check_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * buf);
+static gboolean free_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * buf);
+static void free_all_buffers (GstMpeg2dec * mpeg2dec);
+
 /*static guint gst_mpeg2dec_signals[LAST_SIGNAL] = { 0 };*/
 
 GType
@@ -246,6 +251,8 @@ gst_mpeg2dec_class_init (GstMpeg2decClass * klass)
 static void
 gst_mpeg2dec_init (GstMpeg2dec * mpeg2dec)
 {
+  int i;
+
   /* create the sink and src pads */
   mpeg2dec->sinkpad =
       gst_pad_new_from_template (gst_static_pad_template_get
@@ -285,6 +292,10 @@ gst_mpeg2dec_init (GstMpeg2dec * mpeg2dec)
   mpeg2_accel (MPEG2_ACCEL_DETECT);
   mpeg2dec->closed = TRUE;
   mpeg2dec->have_fbuf = FALSE;
+  mpeg2dec->offset = 0;
+
+  for (i = 0; i < GST_MPEG2DEC_NUM_BUFS; i++)
+    mpeg2dec->buffers[i] = NULL;
 
   GST_FLAG_SET (GST_ELEMENT (mpeg2dec), GST_ELEMENT_EVENT_AWARE);
 }
@@ -294,6 +305,7 @@ gst_mpeg2dec_close_decoder (GstMpeg2dec * mpeg2dec)
 {
   if (!mpeg2dec->closed) {
     mpeg2_close (mpeg2dec->decoder);
+    free_all_buffers (mpeg2dec);
     mpeg2dec->closed = TRUE;
     mpeg2dec->decoder = NULL;
   }
@@ -306,6 +318,7 @@ gst_mpeg2dec_open_decoder (GstMpeg2dec * mpeg2dec)
   mpeg2dec->decoder = mpeg2_init ();
   mpeg2dec->closed = FALSE;
   mpeg2dec->have_fbuf = FALSE;
+  mpeg2_custom_fbuf (mpeg2dec->decoder, 1);
 }
 
 static void
@@ -318,6 +331,21 @@ gst_mpeg2dec_dispose (GObject * object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+static void
+gst_mpeg2dec_reset (GstMpeg2dec * mpeg2dec)
+{
+  /* reset the initial video state */
+  mpeg2dec->format = MPEG2DEC_FORMAT_NONE;
+  mpeg2dec->width = -1;
+  mpeg2dec->height = -1;
+  mpeg2dec->segment_start = 0;
+  mpeg2dec->segment_end = -1;
+  mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
+  mpeg2dec->frame_period = 0;
+  gst_pad_set_explicit_caps (mpeg2dec->srcpad, NULL);
+  gst_mpeg2dec_open_decoder (mpeg2dec);
+  mpeg2dec->need_sequence = TRUE;
+}
 static void
 gst_mpeg2dec_set_index (GstElement * element, GstIndex * index)
 {
@@ -336,20 +364,153 @@ gst_mpeg2dec_get_index (GstElement * element)
   return mpeg2dec->index;
 }
 
+static gboolean
+put_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * buf)
+{
+  int i;
+
+  for (i = 0; i < GST_MPEG2DEC_NUM_BUFS; i++) {
+    if (mpeg2dec->buffers[i] == NULL) {
+      GST_DEBUG_OBJECT (mpeg2dec, "Placing %p at slot %d", buf, i);
+      mpeg2dec->buffers[i] = buf;
+      return TRUE;
+    }
+  }
+  return FALSE;
+}
+
+static gboolean
+check_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * buf)
+{
+  int i;
+
+  for (i = 0; i < GST_MPEG2DEC_NUM_BUFS; i++)
+    if (mpeg2dec->buffers[i] == buf)
+      return TRUE;
+
+  return FALSE;
+}
+
+static gboolean
+free_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * buf)
+{
+  int i;
+
+  for (i = 0; i < GST_MPEG2DEC_NUM_BUFS; i++) {
+    if (mpeg2dec->buffers[i] == buf) {
+      GST_DEBUG_OBJECT (mpeg2dec, "Releasing %p at slot %d", buf, i);
+      gst_buffer_unref (buf);
+      mpeg2dec->buffers[i] = NULL;
+
+      return TRUE;
+    }
+  }
+  return FALSE;
+}
+
+static void
+free_all_buffers (GstMpeg2dec * mpeg2dec)
+{
+  int i;
+
+  for (i = 0; i < GST_MPEG2DEC_NUM_BUFS; i++) {
+    if (mpeg2dec->buffers[i] != NULL) {
+      GST_DEBUG_OBJECT (mpeg2dec, "free_all Releasing %p at slot %d",
+          mpeg2dec->buffers[i], i);
+      gst_buffer_unref (mpeg2dec->buffers[i]);
+      mpeg2dec->buffers[i] = NULL;
+    }
+  }
+}
+
 static GstBuffer *
-gst_mpeg2dec_alloc_buffer (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info,
-    gint64 offset)
+crop_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * input)
+{
+  unsigned char *in_data;
+  unsigned char *out_data;
+  unsigned int h_subsample;
+  unsigned int v_subsample;
+  unsigned int line;
+  GstBuffer *outbuf = input;
+
+  /*We crop only if the target region is smaller than the input one */
+  if ((mpeg2dec->decoded_width > mpeg2dec->width) ||
+      (mpeg2dec->decoded_height > mpeg2dec->height)) {
+    /* If we don't know about the format, we just return the original 
+     * buffer.
+     */
+    if (mpeg2dec->format == MPEG2DEC_FORMAT_I422 ||
+        mpeg2dec->format == MPEG2DEC_FORMAT_I420 ||
+        mpeg2dec->format == MPEG2DEC_FORMAT_YV12) {
+      /*FIXME:  I have tried to use gst_buffer_copy_on_write, but it 
+       *        still have some artifact, so I'me allocating new buffer
+       *        for each frame decoded...
+       */
+      if (mpeg2dec->format == MPEG2DEC_FORMAT_I422) {
+        outbuf =
+            gst_buffer_new_and_alloc (mpeg2dec->width * mpeg2dec->height * 2);
+        h_subsample = 2;
+        v_subsample = 1;
+      } else {
+        outbuf =
+            gst_buffer_new_and_alloc (mpeg2dec->width * mpeg2dec->height * 1.5);
+        h_subsample = 2;
+        v_subsample = 2;
+      }
+
+      GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (input);
+      GST_BUFFER_OFFSET (outbuf) = GST_BUFFER_OFFSET (input);
+      GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (input);
+
+      /* Copy Y first */
+      in_data = GST_BUFFER_DATA (input);
+      out_data = GST_BUFFER_DATA (outbuf);
+      for (line = 0; line < mpeg2dec->height; line++) {
+        memcpy (out_data, in_data, mpeg2dec->width);
+        out_data += mpeg2dec->width;
+        in_data += mpeg2dec->decoded_width;
+      }
+
+      /* Now copy U & V */
+      in_data =
+          GST_BUFFER_DATA (input) +
+          mpeg2dec->decoded_width * mpeg2dec->decoded_height;
+      for (line = 0; line < mpeg2dec->height / v_subsample; line++) {
+        memcpy (out_data, in_data, mpeg2dec->width / h_subsample);
+        memcpy (out_data +
+            mpeg2dec->width * mpeg2dec->height / (v_subsample * h_subsample),
+            in_data +
+            mpeg2dec->decoded_width * mpeg2dec->decoded_height / (v_subsample *
+                h_subsample), mpeg2dec->width / h_subsample);
+        out_data += mpeg2dec->width / h_subsample;
+        in_data += mpeg2dec->decoded_width / h_subsample;
+      }
+
+      gst_buffer_unref (input);
+    }
+  }
+
+  return outbuf;
+}
+
+static GstBuffer *
+gst_mpeg2dec_alloc_buffer (GstMpeg2dec * mpeg2dec, gint64 offset)
 {
   GstBuffer *outbuf = NULL;
-  gint size = mpeg2dec->width * mpeg2dec->height;
+  gint size = mpeg2dec->decoded_width * mpeg2dec->decoded_height;
   guint8 *buf[3], *out;
-  const mpeg2_picture_t *picture;
 
   if (mpeg2dec->format == MPEG2DEC_FORMAT_I422) {
     outbuf =
         gst_pad_alloc_buffer (mpeg2dec->srcpad, GST_BUFFER_OFFSET_NONE,
         size * 2);
 
+    if (!outbuf) {
+      GST_ELEMENT_ERROR (mpeg2dec, RESOURCE, FAILED, (NULL),
+          ("Failed to allocate memory for buffer"));
+      return NULL;
+    }
+
     out = GST_BUFFER_DATA (outbuf);
 
     buf[0] = out;
@@ -375,19 +536,20 @@ gst_mpeg2dec_alloc_buffer (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info,
     }
   }
 
-  gst_buffer_ref (outbuf);
+  if (!put_buffer (mpeg2dec, outbuf)) {
+#if 0
+    GST_ELEMENT_ERROR (mpeg2dec, LIBRARY, TOO_LAZY, (NULL),
+        ("No free slot. libmpeg2 did not discard buffers."));
+#else
+    GST_WARNING_OBJECT (mpeg2dec,
+        "No free slot. libmpeg2 did not discard buffers.");
+#endif
+    return NULL;
+  }
 
   mpeg2_custom_fbuf (mpeg2dec->decoder, 1);
   mpeg2_set_buf (mpeg2dec->decoder, buf, outbuf);
 
-  picture = info->current_picture;
-
-  if (picture
-      && (picture->flags & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_I) {
-    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
-  } else {
-    GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_KEY_UNIT);
-  }
   /* we store the original byteoffset of this picture in the stream here
    * because we need it for indexing */
   GST_BUFFER_OFFSET (outbuf) = offset;
@@ -450,6 +612,222 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec * mpeg2dec)
   return TRUE;
 }
 
+static gboolean
+handle_sequence (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
+{
+  gint i;
+
+  mpeg2dec->width = info->sequence->picture_width;
+  mpeg2dec->height = info->sequence->picture_height;
+  mpeg2dec->pixel_width = info->sequence->pixel_width;
+  mpeg2dec->pixel_height = info->sequence->pixel_height;
+  mpeg2dec->decoded_width = info->sequence->width;
+  mpeg2dec->decoded_height = info->sequence->height;
+  mpeg2dec->total_frames = 0;
+
+  /* find framerate */
+  for (i = 0; i < 9; i++) {
+    if (info->sequence->frame_period == frame_periods[i]) {
+      mpeg2dec->frame_rate = fpss[i];
+    }
+  }
+  mpeg2dec->frame_period = info->sequence->frame_period * GST_USECOND / 27;
+
+  GST_DEBUG_OBJECT (mpeg2dec,
+      "sequence flags: %d, frame period: %d (%g), frame rate: %g",
+      info->sequence->flags, info->sequence->frame_period,
+      (double) (mpeg2dec->frame_period) / GST_SECOND, mpeg2dec->frame_rate);
+  GST_DEBUG_OBJECT (mpeg2dec, "profile: %02x, colour_primaries: %d",
+      info->sequence->profile_level_id, info->sequence->colour_primaries);
+  GST_DEBUG_OBJECT (mpeg2dec, "transfer chars: %d, matrix coef: %d",
+      info->sequence->transfer_characteristics,
+      info->sequence->matrix_coefficients);
+
+  if (!gst_mpeg2dec_negotiate_format (mpeg2dec)) {
+    GST_ELEMENT_ERROR (mpeg2dec, CORE, NEGOTIATION, (NULL), (NULL));
+    return FALSE;
+  }
+
+  free_all_buffers (mpeg2dec);
+  if (!gst_mpeg2dec_alloc_buffer (mpeg2dec, mpeg2dec->offset) ||
+      !gst_mpeg2dec_alloc_buffer (mpeg2dec, mpeg2dec->offset) ||
+      !gst_mpeg2dec_alloc_buffer (mpeg2dec, mpeg2dec->offset))
+    return FALSE;
+
+  mpeg2dec->need_sequence = FALSE;
+
+  return TRUE;
+}
+
+static gboolean
+handle_picture (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
+{
+  gboolean key_frame = FALSE;
+  GstBuffer *outbuf;
+
+  if (info->current_picture) {
+    key_frame =
+        (info->current_picture->flags & PIC_MASK_CODING_TYPE) ==
+        PIC_FLAG_CODING_TYPE_I;
+  }
+  outbuf = gst_mpeg2dec_alloc_buffer (mpeg2dec, mpeg2dec->offset);
+  if (!outbuf)
+    return FALSE;
+
+  GST_DEBUG_OBJECT (mpeg2dec, "picture %s, outbuf %p, offset %"
+      G_GINT64_FORMAT,
+      key_frame ? ", kf," : "    ", outbuf, GST_BUFFER_OFFSET (outbuf)
+      );
+
+  if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_PICTURE && key_frame)
+    mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_KEYFRAME;
+
+  if (!GST_PAD_IS_USABLE (mpeg2dec->srcpad))
+    mpeg2_skip (mpeg2dec->decoder, 1);
+  else
+    mpeg2_skip (mpeg2dec->decoder, 0);
+
+  return TRUE;
+}
+
+static gboolean
+handle_slice (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
+{
+  GstBuffer *outbuf = NULL;
+  gboolean skip = FALSE;
+
+  GST_DEBUG_OBJECT (mpeg2dec, "picture slice/end %p %p %p %p",
+      info->display_fbuf,
+      info->display_picture, info->current_picture,
+      (info->display_fbuf ? info->display_fbuf->id : NULL));
+
+  if (info->display_fbuf && info->display_fbuf->id) {
+    const mpeg2_picture_t *picture;
+    gboolean key_frame = FALSE;
+
+    outbuf = GST_BUFFER (info->display_fbuf->id);
+    if (!check_buffer (mpeg2dec, outbuf)) {
+      GST_ELEMENT_ERROR (mpeg2dec, RESOURCE, FAILED, (NULL),
+          ("libmpeg2 reported invalid buffer %p", outbuf));
+      return FALSE;
+    }
+
+    gst_buffer_ref (outbuf);
+
+    picture = info->display_picture;
+
+    key_frame =
+        (picture->flags & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_I;
+    GST_DEBUG_OBJECT (mpeg2dec, "picture keyframe %d", key_frame);
+
+    if (key_frame) {
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
+    }
+    if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_KEYFRAME && key_frame)
+      mpeg2dec->discont_state = MPEG2DEC_DISC_NONE;
+
+#if MPEG2_RELEASE < MPEG2_VERSION(0,4,0)
+    if (picture->flags & PIC_FLAG_PTS) {
+      GstClockTime time = MPEG_TIME_TO_GST_TIME (picture->pts);
+#else
+    if (picture->flags & PIC_FLAG_TAGS) {
+      GstClockTime time =
+          MPEG_TIME_TO_GST_TIME ((GstClockTime) (picture->
+              tag2) << 32 | picture->tag);
+#endif
+      GST_DEBUG_OBJECT (mpeg2dec,
+          "picture had pts %" GST_TIME_FORMAT ", we had %"
+          GST_TIME_FORMAT, GST_TIME_ARGS (time),
+          GST_TIME_ARGS (mpeg2dec->next_time));
+      GST_BUFFER_TIMESTAMP (outbuf) = mpeg2dec->next_time = time;
+    } else {
+      GST_DEBUG_OBJECT (mpeg2dec,
+          "picture didn't have pts. Using %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (mpeg2dec->next_time));
+      GST_BUFFER_TIMESTAMP (outbuf) = mpeg2dec->next_time;
+    }
+
+    /* TODO set correct offset here based on frame number */
+    if (info->display_picture_2nd) {
+      GST_BUFFER_DURATION (outbuf) = (picture->nb_fields +
+          info->display_picture_2nd->nb_fields) * mpeg2dec->frame_period / 2;
+    } else {
+      GST_BUFFER_DURATION (outbuf) =
+          picture->nb_fields * mpeg2dec->frame_period / 2;
+    }
+    mpeg2dec->next_time += GST_BUFFER_DURATION (outbuf);
+
+    GST_DEBUG_OBJECT (mpeg2dec,
+        "picture: %s %s fields:%d off:%" G_GINT64_FORMAT " ts:%"
+        GST_TIME_FORMAT,
+        (picture->flags & PIC_FLAG_TOP_FIELD_FIRST ? "tff " : "    "),
+        (picture->flags & PIC_FLAG_PROGRESSIVE_FRAME ? "prog" : "    "),
+        picture->nb_fields, GST_BUFFER_OFFSET (outbuf),
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
+
+    if (mpeg2dec->index) {
+      gst_index_add_association (mpeg2dec->index, mpeg2dec->index_id,
+          (key_frame ? GST_ASSOCIATION_FLAG_KEY_UNIT : 0),
+          GST_FORMAT_BYTES, GST_BUFFER_OFFSET (outbuf),
+          GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (outbuf), 0);
+    }
+
+    if (picture->flags & PIC_FLAG_SKIP) {
+      GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer because of skip flag");
+      gst_buffer_unref (outbuf);
+    } else if (!GST_PAD_IS_USABLE (mpeg2dec->srcpad)
+        || !gst_pad_is_negotiated (mpeg2dec->srcpad)) {
+      GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer, pad not usable");
+      gst_buffer_unref (outbuf);
+    } else if (mpeg2dec->discont_state != MPEG2DEC_DISC_NONE) {
+      GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer, discont state %d",
+          mpeg2dec->discont_state);
+      gst_buffer_unref (outbuf);
+    } else if (mpeg2dec->next_time < mpeg2dec->segment_start) {
+      GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer, next_time %"
+          GST_TIME_FORMAT " <  segment_start %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (mpeg2dec->next_time),
+          GST_TIME_ARGS (mpeg2dec->segment_start));
+      gst_buffer_unref (outbuf);
+    } else if (skip) {
+      GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer, asked to skip");
+      gst_buffer_unref (outbuf);
+    } else {
+      GST_LOG_OBJECT (mpeg2dec, "pushing buffer, timestamp %"
+          GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+          GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
+
+      if ((mpeg2dec->decoded_height > mpeg2dec->height) ||
+          (mpeg2dec->decoded_width > mpeg2dec->width))
+        outbuf = crop_buffer (mpeg2dec, outbuf);
+
+      if (info->current_picture
+          && (info->current_picture->flags & PIC_MASK_CODING_TYPE) ==
+          PIC_FLAG_CODING_TYPE_I) {
+        GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
+      } else {
+        GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_KEY_UNIT);
+      }
+      gst_pad_push (mpeg2dec->srcpad, GST_DATA (outbuf));
+    }
+  } else if (info->display_fbuf && !info->display_fbuf->id) {
+    GST_ELEMENT_ERROR (mpeg2dec, LIBRARY, TOO_LAZY, (NULL),
+        ("libmpeg2 reported invalid buffer %p", info->discard_fbuf->id));
+  }
+
+  if (info->discard_fbuf && info->discard_fbuf->id) {
+    if (free_buffer (mpeg2dec, GST_BUFFER (info->discard_fbuf->id))) {
+      GST_DEBUG_OBJECT (mpeg2dec, "Discarded buffer %p",
+          info->discard_fbuf->id);
+    } else {
+      GST_ELEMENT_ERROR (mpeg2dec, LIBRARY, TOO_LAZY, (NULL),
+          ("libmpeg2 reported invalid buffer %p", info->discard_fbuf->id));
+    }
+  }
+  return TRUE;
+}
+
 #if 0
 static void
 update_streaminfo (GstMpeg2dec * mpeg2dec)
@@ -484,24 +862,67 @@ static void
 gst_mpeg2dec_flush_decoder (GstMpeg2dec * mpeg2dec)
 {
   if (mpeg2dec->decoder) {
-#if MPEG2_RELEASE < MPEG2_VERSION(0,4,0)
     const mpeg2_info_t *info = mpeg2_info (mpeg2dec->decoder);
     mpeg2_state_t state;
 
+    /*
+     * iterate the decoder and free buffers
+     */
     do {
       state = mpeg2_parse (mpeg2dec->decoder);
-      if (state == STATE_END) {
-        if (info->discard_fbuf && info->discard_fbuf->id) {
-          gst_buffer_unref ((GstBuffer *) info->discard_fbuf->id);
-        }
+      switch (state) {
+        case STATE_SEQUENCE:
+          if (!handle_sequence (mpeg2dec, info)) {
+            gst_mpeg2dec_close_decoder (mpeg2dec);
+            gst_mpeg2dec_open_decoder (mpeg2dec);
+            state = -1;
+          }
+          break;
+        case STATE_PICTURE:
+          if (!handle_picture (mpeg2dec, info)) {
+            gst_mpeg2dec_close_decoder (mpeg2dec);
+            gst_mpeg2dec_open_decoder (mpeg2dec);
+            state = -1;
+            break;
+          }
+          mpeg2_skip (mpeg2dec->decoder, 1);
+          break;
+        case STATE_END:
+#if MPEG2_RELEASE >= MPEG2_VERSION (0, 4, 0)
+        case STATE_INVALID_END:
+#endif
+        case STATE_SLICE:
+          if (info->discard_fbuf) {
+            if (free_buffer (mpeg2dec, GST_BUFFER (info->discard_fbuf->id))) {
+              GST_DEBUG_OBJECT (mpeg2dec, "Discarded buffer %p",
+                  info->discard_fbuf->id);
+            } else {
+              GST_ELEMENT_ERROR (mpeg2dec, LIBRARY, TOO_LAZY, (NULL),
+                  ("libmpeg2 reported invalid buffer %p, fbuf: %p",
+                      info->discard_fbuf->id, info->discard_fbuf));
+            }
+          }
+          break;
+        case STATE_INVALID:
+          GST_WARNING_OBJECT (mpeg2dec, "Decoding error");
+          /*
+           * We need to close the decoder here, according to docs
+           */
+          gst_mpeg2dec_close_decoder (mpeg2dec);
+          gst_mpeg2dec_open_decoder (mpeg2dec);
+          return;
+        default:
+          break;
       }
     }
     while (state != STATE_BUFFER && state != -1);
-#else
+
+#if MPEG2_RELEASE >= MPEG2_VERSION(0,4,0)
     GST_DEBUG_OBJECT (mpeg2dec, "resetting mpeg2 stream decoder");
     /* 0 starts at next picture, 1 at next sequence header */
     mpeg2_reset (mpeg2dec->decoder, 0);
 #endif
+
   }
 }
 
@@ -512,7 +933,7 @@ gst_mpeg2dec_chain (GstPad * pad, GstData * _data)
   GstMpeg2dec *mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
   guint32 size;
   guint8 *data, *end;
-  gint64 pts;
+  GstClockTime pts;
   const mpeg2_info_t *info;
   mpeg2_state_t state;
   gboolean done = FALSE;
@@ -523,8 +944,32 @@ gst_mpeg2dec_chain (GstPad * pad, GstData * _data)
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_DISCONTINUOUS:
       {
-        GST_DEBUG_OBJECT (mpeg2dec, "discont, resetting next_time to 0");
-        mpeg2dec->next_time = 0;
+        GstClockTime time;
+
+        if (!gst_event_discont_get_value (event, GST_FORMAT_TIME, &time)
+            || !GST_CLOCK_TIME_IS_VALID (time)) {
+          GST_WARNING_OBJECT (mpeg2dec,
+              "No new time offset in discont event %p", event);
+        } else {
+          mpeg2dec->next_time = time;
+          GST_DEBUG_OBJECT (mpeg2dec,
+              "discont, reset next_time to %" G_GUINT64_FORMAT " (%"
+              GST_TIME_FORMAT ")", mpeg2dec->next_time,
+              GST_TIME_ARGS (mpeg2dec->next_time));
+        }
+
+        if (GST_EVENT_DISCONT_NEW_MEDIA (event)) {
+          gst_mpeg2dec_reset (mpeg2dec);
+        } else {
+          mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
+          gst_mpeg2dec_flush_decoder (mpeg2dec);
+        }
+
+        gst_pad_event_default (pad, event);
+        return;
+      }
+      case GST_EVENT_FLUSH:
+      {
         mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
         gst_mpeg2dec_flush_decoder (mpeg2dec);
         gst_pad_event_default (pad, event);
@@ -553,7 +998,7 @@ gst_mpeg2dec_chain (GstPad * pad, GstData * _data)
   info = mpeg2_info (mpeg2dec->decoder);
   end = data + size;
 
-  if (pts != -1) {
+  if (pts != GST_CLOCK_TIME_NONE) {
     gint64 mpeg_pts = GST_TIME_TO_MPEG_TIME (pts);
 
     GST_DEBUG_OBJECT (mpeg2dec,
@@ -570,60 +1015,21 @@ gst_mpeg2dec_chain (GstPad * pad, GstData * _data)
     GST_LOG ("no pts");
   }
 
-  GST_LOG ("calling mpeg2_buffer");
+  GST_LOG_OBJECT (mpeg2dec, "calling mpeg2_buffer");
   mpeg2_buffer (mpeg2dec->decoder, data, end);
-  GST_LOG ("calling mpeg2_buffer done");
+  GST_LOG_OBJECT (mpeg2dec, "calling mpeg2_buffer done");
 
+  mpeg2dec->offset = GST_BUFFER_OFFSET (buf);
   while (!done) {
-    gboolean slice = FALSE;
-
-    GST_LOG ("calling parse");
+    GST_LOG_OBJECT (mpeg2dec, "calling parse");
     state = mpeg2_parse (mpeg2dec->decoder);
     GST_DEBUG_OBJECT (mpeg2dec, "parse state %d", state);
     switch (state) {
       case STATE_SEQUENCE:
-      {
-        gint i;
-
-        mpeg2dec->width = info->sequence->width;
-        mpeg2dec->height = info->sequence->height;
-        mpeg2dec->pixel_width = info->sequence->pixel_width;
-        mpeg2dec->pixel_height = info->sequence->pixel_height;
-        mpeg2dec->total_frames = 0;
-
-        /* find framerate */
-        for (i = 0; i < 9; i++) {
-          if (info->sequence->frame_period == frame_periods[i]) {
-            mpeg2dec->frame_rate = fpss[i];
-          }
-        }
-        mpeg2dec->frame_period =
-            info->sequence->frame_period * GST_USECOND / 27;
-
-        GST_DEBUG_OBJECT (mpeg2dec,
-            "sequence flags: %d, frame period: %d, frame rate: %d",
-            info->sequence->flags, info->sequence->frame_period,
-            mpeg2dec->frame_rate);
-        GST_DEBUG_OBJECT (mpeg2dec, "profile: %02x, colour_primaries: %d",
-            info->sequence->profile_level_id, info->sequence->colour_primaries);
-        GST_DEBUG_OBJECT (mpeg2dec, "transfer chars: %d, matrix coef: %d",
-            info->sequence->transfer_characteristics,
-            info->sequence->matrix_coefficients);
-
-        if (!gst_mpeg2dec_negotiate_format (mpeg2dec)) {
-          GST_ELEMENT_ERROR (mpeg2dec, CORE, NEGOTIATION, (NULL), (NULL));
+        if (!handle_sequence (mpeg2dec, info)) {
           goto exit;
         }
 
-        if (!mpeg2dec->have_fbuf) {
-          /* alloc 3 buffers */
-          gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
-          gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
-          gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
-          mpeg2dec->have_fbuf = TRUE;
-        }
-
-        mpeg2dec->need_sequence = FALSE;
         if (mpeg2dec->pending_event) {
           done =
               GST_EVENT_SEEK_FLAGS (mpeg2dec->
@@ -633,170 +1039,48 @@ gst_mpeg2dec_chain (GstPad * pad, GstData * _data)
           mpeg2dec->pending_event = NULL;
         }
         break;
-      }
       case STATE_SEQUENCE_REPEATED:
         GST_DEBUG_OBJECT (mpeg2dec, "sequence repeated");
       case STATE_GOP:
         break;
       case STATE_PICTURE:
       {
-        gboolean key_frame = FALSE;
-        GstBuffer *outbuf;
-
-        if (info->current_picture) {
-          key_frame =
-              (info->current_picture->flags & PIC_MASK_CODING_TYPE) ==
-              PIC_FLAG_CODING_TYPE_I;
+        if (!handle_picture (mpeg2dec, info)) {
+          goto exit;
         }
-        outbuf =
-            gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
-
-        GST_DEBUG_OBJECT (mpeg2dec, "picture %s, outbuf %p, offset %"
-            G_GINT64_FORMAT ", pts %" G_GINT64_FORMAT,
-            key_frame ? ", kf," : "    ", outbuf, GST_BUFFER_OFFSET (outbuf),
-            pts);
-
-        if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_PICTURE && key_frame)
-          mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_KEYFRAME;
-
-        if (!GST_PAD_IS_USABLE (mpeg2dec->srcpad))
-          mpeg2_skip (mpeg2dec->decoder, 1);
-        else
-          mpeg2_skip (mpeg2dec->decoder, 0);
         break;
       }
       case STATE_SLICE_1ST:
-        GST_DEBUG_OBJECT (mpeg2dec, "slice 1st");
+        GST_LOG_OBJECT (mpeg2dec, "1st slice of frame encountered");
         break;
       case STATE_PICTURE_2ND:
-        GST_DEBUG_OBJECT (mpeg2dec, "picture second");
+        GST_LOG_OBJECT (mpeg2dec,
+            "Second picture header encountered. Decoding 2nd field");
         break;
-      case STATE_SLICE:
-        slice = TRUE;
 #if MPEG2_RELEASE >= MPEG2_VERSION (0, 4, 0)
       case STATE_INVALID_END:
 #endif
       case STATE_END:
-      {
-        GstBuffer *outbuf = NULL;
-        gboolean skip = FALSE;
-
-        if (!slice) {
-          mpeg2dec->need_sequence = TRUE;
-        }
-        GST_DEBUG_OBJECT (mpeg2dec, "picture slice/end %p %p %p %p",
-            info->display_fbuf,
-            info->display_picture, info->current_picture,
-            (info->display_fbuf ? info->display_fbuf->id : NULL));
-
-        if (info->display_fbuf && info->display_fbuf->id) {
-          const mpeg2_picture_t *picture;
-          gboolean key_frame = FALSE;
-
-          outbuf = (GstBuffer *) info->display_fbuf->id;
-          picture = info->display_picture;
-
-          key_frame =
-              (picture->flags & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_I;
-          GST_DEBUG_OBJECT (mpeg2dec, "picture keyframe %d", key_frame);
-
-          if (key_frame) {
-            GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
-          }
-          if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_KEYFRAME
-              && key_frame)
-            mpeg2dec->discont_state = MPEG2DEC_DISC_NONE;
-
-#if MPEG2_RELEASE < MPEG2_VERSION(0,4,0)
-          if (picture->flags & PIC_FLAG_PTS) {
-            GstClockTime time = MPEG_TIME_TO_GST_TIME (picture->pts);
-#else
-          if (picture->flags & PIC_FLAG_TAGS) {
-            GstClockTime time =
-                MPEG_TIME_TO_GST_TIME ((guint64) picture->tag2 << 32 | picture->
-                tag);
-#endif
-
-            GST_DEBUG_OBJECT (mpeg2dec, "picture had pts %" GST_TIME_FORMAT,
-                GST_TIME_ARGS (time));
-            GST_BUFFER_TIMESTAMP (outbuf) = mpeg2dec->next_time = time;
-          } else {
-            GST_DEBUG_OBJECT (mpeg2dec,
-                "picture didn't have pts using %" GST_TIME_FORMAT,
-                GST_TIME_ARGS (mpeg2dec->next_time));
-            GST_BUFFER_TIMESTAMP (outbuf) = mpeg2dec->next_time;
-          }
-
-          mpeg2dec->next_time +=
-              (mpeg2dec->frame_period * picture->nb_fields) >> 1;
-
-          GST_DEBUG_OBJECT (mpeg2dec,
-              "picture: %s %s fields:%d off:%" G_GINT64_FORMAT " ts:%"
-              GST_TIME_FORMAT,
-              (picture->flags & PIC_FLAG_TOP_FIELD_FIRST ? "tff " : "    "),
-              (picture->flags & PIC_FLAG_PROGRESSIVE_FRAME ? "prog" : "    "),
-              picture->nb_fields, GST_BUFFER_OFFSET (outbuf),
-              GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
-
-          if (mpeg2dec->index) {
-            gst_index_add_association (mpeg2dec->index, mpeg2dec->index_id,
-                (key_frame ? GST_ASSOCIATION_FLAG_KEY_UNIT : 0),
-                GST_FORMAT_BYTES, GST_BUFFER_OFFSET (outbuf),
-                GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (outbuf), 0);
-          }
-
-          if (picture->flags & PIC_FLAG_SKIP) {
-            GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer because of skip flag");
-            gst_buffer_unref (outbuf);
-          } else if (!GST_PAD_IS_USABLE (mpeg2dec->srcpad)) {
-            GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer, pad not usable");
-            gst_buffer_unref (outbuf);
-          } else if (mpeg2dec->discont_state != MPEG2DEC_DISC_NONE) {
-            GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer, discont state %d",
-                mpeg2dec->discont_state);
-            gst_buffer_unref (outbuf);
-          } else if (mpeg2dec->next_time < mpeg2dec->segment_start) {
-            GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer, next_time %"
-                GST_TIME_FORMAT " <  segment_start %" GST_TIME_FORMAT,
-                GST_TIME_ARGS (mpeg2dec->next_time),
-                GST_TIME_ARGS (mpeg2dec->segment_start));
-            gst_buffer_unref (outbuf);
-          } else if (skip) {
-            GST_DEBUG_OBJECT (mpeg2dec, "dropping buffer, asked to skip");
-            gst_buffer_unref (outbuf);
-          } else {
-            /* TODO set correct offset here based on frame number */
-            GST_BUFFER_DURATION (outbuf) = mpeg2dec->frame_period;
-            GST_LOG_OBJECT (mpeg2dec, "pushing buffer, timestamp %"
-                GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT,
-                GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
-                GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
-            gst_pad_push (mpeg2dec->srcpad, GST_DATA (outbuf));
-          }
-        } else if (info->display_fbuf && !info->display_fbuf->id) {
-          GST_WARNING ("Got a frame from libmpeg2, but it has no buffer");
+        mpeg2dec->need_sequence = TRUE;
+      case STATE_SLICE:
+        if (!handle_slice (mpeg2dec, info)) {
+          goto exit;
         }
 
-        if (info->discard_fbuf && info->discard_fbuf->id) {
-          gst_buffer_unref ((GstBuffer *) info->discard_fbuf->id);
-        }
         break;
-      }
-        /* need more data */
       case STATE_BUFFER:
       case -1:
+        /* need more data */
         done = TRUE;
         break;
         /* error */
       case STATE_INVALID:
-        g_warning ("mpeg2dec: decoding error");
-        /* it looks like setting a new frame in libmpeg2 avoids a crash */
-        /* FIXME figure out how this screws up sync and buffer leakage */
-        gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
+        GST_WARNING_OBJECT (mpeg2dec, "Decoding error");
+        goto exit;
+
         break;
       default:
-        g_warning ("%s: unhandled state %d, FIXME",
-            gst_element_get_name (GST_ELEMENT (mpeg2dec)), state);
+        GST_ERROR_OBJECT (mpeg2dec, "Unknown libmpeg2 state %d, FIXME", state);
         break;
     }
 
@@ -815,7 +1099,16 @@ gst_mpeg2dec_chain (GstPad * pad, GstData * _data)
     }
 #endif
   }
+  gst_buffer_unref (buf);
+  return;
+
 exit:
+  /*
+   * Close and reopen the decoder, because
+   * something went pretty wrong
+   */
+  gst_mpeg2dec_close_decoder (mpeg2dec);
+  gst_mpeg2dec_open_decoder (mpeg2dec);
   gst_buffer_unref (buf);
 }
 
@@ -962,7 +1255,7 @@ gst_mpeg2dec_src_query (GstPad * pad, GstQueryType type,
 {
   gboolean res = TRUE;
   GstMpeg2dec *mpeg2dec;
-  static const GstFormat *formats;
+  static const GstFormat *formats = NULL;
 
   mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
 
@@ -977,10 +1270,11 @@ gst_mpeg2dec_src_query (GstPad * pad, GstQueryType type,
           res = FALSE;
 
           /* get our peer formats */
-          formats = gst_pad_get_formats (GST_PAD_PEER (mpeg2dec->sinkpad));
+          if (GST_PAD_PEER (mpeg2dec->sinkpad))
+            formats = gst_pad_get_formats (GST_PAD_PEER (mpeg2dec->sinkpad));
 
           /* while we did not exhaust our seek formats without result */
-          while (formats && *formats) {
+          while (formats && *formats && !res) {
             GstFormat peer_format;
             gint64 peer_value;
 
@@ -1052,7 +1346,7 @@ index_seek (GstPad * pad, GstEvent * event)
       GST_INDEX_LOOKUP_BEFORE, GST_ASSOCIATION_FLAG_KEY_UNIT,
       GST_EVENT_SEEK_FORMAT (event), GST_EVENT_SEEK_OFFSET (event));
 
-  if (entry) {
+  if ((entry) && GST_PAD_PEER (mpeg2dec->sinkpad)) {
     const GstFormat *peer_formats, *try_formats;
 
     /* since we know the exact byteoffset of the frame, make sure to seek on bytes first */
@@ -1084,8 +1378,9 @@ index_seek (GstPad * pad, GstEvent * event)
         /* do the seek */
         if (gst_pad_send_event (GST_PAD_PEER (mpeg2dec->sinkpad), seek_event)) {
           /* seek worked, we're done, loop will exit */
-          gst_mpeg2dec_flush_decoder (mpeg2dec);
+#if 0
           mpeg2dec->segment_start = GST_EVENT_SEEK_OFFSET (event);
+#endif
           return TRUE;
         }
       }
@@ -1102,7 +1397,7 @@ normal_seek (GstPad * pad, GstEvent * event)
   gint64 src_offset;
   gboolean flush;
   GstFormat format;
-  const GstFormat *peer_formats;
+  const GstFormat *peer_formats = NULL;
   gboolean res = FALSE;
   GstMpeg2dec *mpeg2dec;
 
@@ -1122,7 +1417,8 @@ normal_seek (GstPad * pad, GstEvent * event)
   flush = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
 
   /* get our peer formats */
-  peer_formats = gst_pad_get_formats (GST_PAD_PEER (mpeg2dec->sinkpad));
+  if (GST_PAD_PEER (mpeg2dec->sinkpad))
+    peer_formats = gst_pad_get_formats (GST_PAD_PEER (mpeg2dec->sinkpad));
 
   /* while we did not exhaust our seek formats without result */
   while (peer_formats && *peer_formats) {
@@ -1142,7 +1438,9 @@ normal_seek (GstPad * pad, GstEvent * event)
       /* do the seekk */
       if (gst_pad_send_event (GST_PAD_PEER (mpeg2dec->sinkpad), seek_event)) {
         /* seek worked, we're done, loop will exit */
+#if 0
         mpeg2dec->segment_start = GST_EVENT_SEEK_OFFSET (event);
+#endif
         res = TRUE;
         break;
       }
@@ -1188,7 +1486,11 @@ gst_mpeg2dec_src_event (GstPad * pad, GstEvent * event)
       break;
     case GST_EVENT_NAVIGATION:
       /* Forward a navigation event unchanged */
-      return gst_pad_send_event (GST_PAD_PEER (mpeg2dec->sinkpad), event);
+      if (GST_PAD_PEER (mpeg2dec->sinkpad))
+        return gst_pad_send_event (GST_PAD_PEER (mpeg2dec->sinkpad), event);
+
+      res = FALSE;
+      break;
     default:
       res = FALSE;
       break;
@@ -1209,16 +1511,7 @@ gst_mpeg2dec_change_state (GstElement * element)
     {
       mpeg2dec->next_time = 0;
 
-      /* reset the initial video state */
-      mpeg2dec->format = MPEG2DEC_FORMAT_NONE;
-      mpeg2dec->width = -1;
-      mpeg2dec->height = -1;
-      mpeg2dec->segment_start = 0;
-      mpeg2dec->segment_end = -1;
-      mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
-      mpeg2dec->frame_period = 0;
-      gst_mpeg2dec_open_decoder (mpeg2dec);
-      mpeg2dec->need_sequence = TRUE;
+      gst_mpeg2dec_reset (mpeg2dec);
       break;
     }
     case GST_STATE_PAUSED_TO_PLAYING:
@@ -1272,7 +1565,7 @@ gst_mpeg2dec_get_property (GObject * object, guint prop_id, GValue * value,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "mpeg2dec", GST_RANK_PRIMARY,
+  if (!gst_element_register (plugin, "mpeg2dec", GST_RANK_SECONDARY,
           GST_TYPE_MPEG2DEC))
     return FALSE;
 
