@@ -38,8 +38,10 @@ G_BEGIN_DECLS
 #define GST_IS_MPEG2DEC_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_MPEG2DEC))
 
-#define MPEGTIME_TO_GSTTIME(time) (((time) * (GST_MSECOND/10)) / 9LL)
-#define GSTTIME_TO_MPEGTIME(time) (((time) * 9LL) / (GST_MSECOND/10))
+#define MPEG_TIME_TO_GST_TIME(time) (((time) * (GST_MSECOND/10)) / 9LL)
+#define GST_TIME_TO_MPEG_TIME(time) (((time) * 9LL) / (GST_MSECOND/10))
+
+#define GST_MPEG2DEC_NUM_BUFS 6
 
 typedef struct _GstMpeg2dec GstMpeg2dec;
 typedef struct _GstMpeg2decClass GstMpeg2decClass;
@@ -71,9 +73,12 @@ struct _GstMpeg2dec {
   gboolean	 closed;
   gboolean	 have_fbuf;
 
-  /* the timestamp of the next frame */
+  GstBuffer  *buffers[GST_MPEG2DEC_NUM_BUFS];
+
   DiscontState	 discont_state;
-  gint64	 next_time;
+
+  /* the timestamp of the next frame */
+  GstClockTime   next_time;
   gint64	 segment_start;
   gint64	 segment_end;
 
@@ -81,11 +86,15 @@ struct _GstMpeg2dec {
   Mpeg2decFormat format;
   gint		 width;
   gint		 height;
+  gint		 decoded_width;
+  gint		 decoded_height;
   gint		 pixel_width;
   gint		 pixel_height;
   gint		 frame_rate_code;
   gint64	 total_frames;
   gint64	 frame_period;
+  
+  guint64  offset;
   gdouble	 frame_rate;
   gboolean	 need_sequence;
 
