@@ -94,6 +94,10 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
 
     case GST_MAKE_FOURCC ('H', 'F', 'Y', 'U'):
       caps = gst_caps_new_simple ("video/x-huffyuv", NULL);
+      if (strf) {
+        gst_caps_set_simple (caps, "bpp",
+            G_TYPE_INT, (int) strf->bit_cnt, NULL);
+      }
       if (codec_name)
         *codec_name = g_strdup ("Huffman Lossless Codec");
       break;
@@ -136,6 +140,7 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('M', '2', '6', '3'):
+    case GST_MAKE_FOURCC ('m', '2', '6', '3'):
       caps = gst_caps_new_simple ("video/x-h263", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Microsoft H.263");
@@ -159,6 +164,25 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
         *codec_name = g_strdup ("Xirlink H.263");
       break;
 
+      /* apparently not standard H.263...? */
+    case GST_MAKE_FOURCC ('I', '2', '6', '3'):
+      caps = gst_caps_new_simple ("video/x-intel-h263", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Intel H.263");
+      break;
+
+    case GST_MAKE_FOURCC ('h', '2', '6', '4'):
+      caps = gst_caps_new_simple ("video/x-h264", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("ITU H.264");
+      break;
+
+    case GST_MAKE_FOURCC ('V', 'S', 'S', 'H'):
+      caps = gst_caps_new_simple ("video/x-h264", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("VideoSoft H.264");
+      break;
+
     case GST_MAKE_FOURCC ('D', 'I', 'V', '3'):
     case GST_MAKE_FOURCC ('d', 'i', 'v', '3'):
     case GST_MAKE_FOURCC ('D', 'I', 'V', '4'):
@@ -181,6 +205,13 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
         *codec_name = g_strdup ("DivX MPEG-4 Version 4");
       break;
 
+    case GST_MAKE_FOURCC ('B', 'L', 'Z', '0'):
+      caps = gst_caps_new_simple ("video/x-divx",
+          "divxversion", G_TYPE_INT, 4, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Blizzard DivX");
+      break;
+
     case GST_MAKE_FOURCC ('D', 'X', '5', '0'):
       caps = gst_caps_new_simple ("video/x-divx",
           "divxversion", G_TYPE_INT, 5, NULL);
@@ -297,6 +328,13 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
         *codec_name = g_strdup ("Xan Wing Commander 4");
       break;
 
+    case GST_MAKE_FOURCC ('I', 'V', '5', '0'):
+      caps = gst_caps_new_simple ("video/x-intel",
+          "ivversion", G_TYPE_INT, 5, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Intel Video 5");
+      break;
+
     default:
       GST_WARNING ("Unknown video fourcc " GST_FOURCC_FORMAT,
           GST_FOURCC_ARGS (codec_fcc));
@@ -362,7 +400,7 @@ gst_riff_create_audio_caps_with_data (guint16 codec_id,
 {
   gboolean block_align = FALSE, rate_chan = TRUE;
   GstCaps *caps = NULL;
-  gint rate_min = 8000, rate_max = 96000;
+  gint rate_min = 1000, rate_max = 96000;
   gint channels_max = 2;
 
   switch (codec_id) {
@@ -420,6 +458,15 @@ gst_riff_create_audio_caps_with_data (guint16 codec_id,
       if (strf != NULL && strf->size != 8) {
         GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
             strf->size);
+        strf->size = 8;
+        strf->av_bps = 8;
+        strf->blockalign = strf->av_bps * strf->channels;
+      }
+      if (strf != NULL && (strf->av_bps == 0 || strf->blockalign == 0)) {
+        GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
+            strf->av_bps, strf->blockalign);
+        strf->av_bps = strf->size;
+        strf->blockalign = strf->av_bps * strf->channels;
       }
       caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
       if (codec_name)
@@ -430,6 +477,15 @@ gst_riff_create_audio_caps_with_data (guint16 codec_id,
       if (strf != NULL && strf->size != 8) {
         GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
             strf->size);
+        strf->size = 8;
+        strf->av_bps = 8;
+        strf->blockalign = strf->av_bps * strf->channels;
+      }
+      if (strf != NULL && (strf->av_bps == 0 || strf->blockalign == 0)) {
+        GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
+            strf->av_bps, strf->blockalign);
+        strf->av_bps = strf->size;
+        strf->blockalign = strf->av_bps * strf->channels;
       }
       caps = gst_caps_new_simple ("audio/x-alaw", NULL);
       if (codec_name)
@@ -459,6 +515,8 @@ gst_riff_create_audio_caps_with_data (guint16 codec_id,
     {
       gint version = (codec_id - GST_RIFF_WAVE_FORMAT_WMAV1) + 1;
 
+      channels_max = 6;
+
       block_align = TRUE;
 
       caps = gst_caps_new_simple ("audio/x-wma",
@@ -505,7 +563,7 @@ gst_riff_create_audio_caps_with_data (guint16 codec_id,
     }
     if (block_align) {
       gst_caps_set_simple (caps,
-          "block_align", GST_TYPE_INT_RANGE, 1, 8192, NULL);
+          "block_align", GST_TYPE_INT_RANGE, 1, G_MAXINT, NULL);
     }
   }
 
@@ -565,6 +623,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('D', 'V', 'S', 'D'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '1'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '2'),
+    GST_MAKE_FOURCC ('W', 'M', 'V', '3'),
     GST_MAKE_FOURCC ('M', 'P', 'G', '4'),
     GST_MAKE_FOURCC ('M', 'P', '4', '2'),
     GST_MAKE_FOURCC ('M', 'P', '4', '3'),
@@ -572,6 +631,8 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('D', 'I', 'V', '3'),
     GST_MAKE_FOURCC ('M', 'P', 'E', 'G'),
     GST_MAKE_FOURCC ('H', '2', '6', '3'),
+    GST_MAKE_FOURCC ('I', '2', '6', '3'),
+    GST_MAKE_FOURCC ('h', '2', '6', '4'),
     GST_MAKE_FOURCC ('D', 'I', 'V', 'X'),
     GST_MAKE_FOURCC ('D', 'X', '5', '0'),
     GST_MAKE_FOURCC ('X', 'V', 'I', 'D'),
@@ -581,6 +642,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('R', 'L', 'E', ' '),
     GST_MAKE_FOURCC ('D', 'I', 'B', ' '),
     GST_MAKE_FOURCC ('X', 'x', 'a', 'n'),
+    GST_MAKE_FOURCC ('I', 'V', '5', '0'),
     /* FILL ME */
     0
   };
@@ -612,6 +674,7 @@ gst_riff_create_audio_template_caps (void)
     GST_RIFF_WAVE_FORMAT_DVI_ADPCM,
     GST_RIFF_WAVE_FORMAT_WMAV1,
     GST_RIFF_WAVE_FORMAT_WMAV2,
+    GST_RIFF_WAVE_FORMAT_WMAV3,
     /* FILL ME */
     0
   };
