@@ -190,7 +190,7 @@ gst_riff_read_use_event (GstRiffRead * riff, GstEvent * event)
     default:
       GST_WARNING ("don't know how to handle event %d", GST_EVENT_TYPE (event));
       gst_pad_event_default (riff->sinkpad, event);
-      return FALSE;
+      return TRUE;
   }
 
   /* happy */
@@ -260,7 +260,7 @@ gst_riff_peek_head (GstRiffRead * riff,
  */
 
 GstBuffer *
-gst_riff_read_element_data (GstRiffRead * riff, guint length, guint * got_bytes)
+gst_riff_peek_element_data (GstRiffRead * riff, guint length, guint * got_bytes)
 {
   GstBuffer *buf = NULL;
   guint32 got;
@@ -272,14 +272,28 @@ gst_riff_read_element_data (GstRiffRead * riff, guint length, guint * got_bytes)
       return NULL;
   }
 
+  if (got_bytes)
+    *got_bytes = got;
+
+  return buf;
+}
+
+GstBuffer *
+gst_riff_read_element_data (GstRiffRead * riff, guint length, guint * got_bytes)
+{
+  GstBuffer *buf;
+
+  if (!(buf = gst_riff_peek_element_data (riff, length, got_bytes)))
+    return NULL;
+
   /* we need 16-bit alignment */
   if (length & 1)
     length++;
 
-  gst_bytestream_flush (riff->bs, length);
-
-  if (got_bytes)
-    *got_bytes = got;
+  if (!gst_bytestream_flush (riff->bs, length)) {
+    gst_buffer_unref (buf);
+    return NULL;
+  }
 
   return buf;
 }
@@ -586,6 +600,9 @@ gst_riff_read_strf_vids_with_data (GstRiffRead * riff,
     if (len > 0) {
       *extradata = gst_buffer_create_sub (buf, strf->size, len);
     }
+  } else if (strf->size > sizeof (gst_riff_strf_vids)) {
+    *extradata = gst_buffer_create_sub (buf,
+        sizeof (gst_riff_strf_vids), strf->size - sizeof (gst_riff_strf_vids));
   }
 
   /* debug */
