@@ -31,6 +31,8 @@
 GST_DEBUG_CATEGORY_STATIC (avidemux_debug);
 #define GST_CAT_DEFAULT avidemux_debug
 
+GST_DEBUG_CATEGORY_EXTERN (GST_CAT_EVENT);
+
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -512,15 +514,19 @@ gst_avi_demux_handle_src_event (GstPad * pad, GstEvent * event)
   GstAviDemux *avi = GST_AVI_DEMUX (gst_pad_get_parent (pad));
   avi_stream_context *stream;
 
-  if (!avi->index_entries)
+  GST_CAT_DEBUG_OBJECT (GST_CAT_EVENT, avi,
+      "have event type %d: %p on src pad", GST_EVENT_TYPE (event), event);
+  if (!avi->index_entries) {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_EVENT, avi, "no index entries, returning");
     return FALSE;
+  }
 
   stream = gst_pad_get_element_private (pad);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-      GST_DEBUG ("seek format %d, %08x", GST_EVENT_SEEK_FORMAT (event),
-          stream->strh->type);
+      GST_DEBUG_OBJECT (avi, "seek format %d, %08x",
+          GST_EVENT_SEEK_FORMAT (event), stream->strh->type);
 
       switch (GST_EVENT_SEEK_FORMAT (event)) {
         case GST_FORMAT_BYTES:
@@ -530,12 +536,8 @@ gst_avi_demux_handle_src_event (GstPad * pad, GstEvent * event)
           gint64 desired_offset = GST_EVENT_SEEK_OFFSET (event);
           guint32 flags;
 
-          /* no seek on audio yet */
-          if (stream->strh->type == GST_RIFF_FCC_auds) {
-            res = FALSE;
-            goto done;
-          }
-          GST_DEBUG ("seeking to %" G_GINT64_FORMAT, desired_offset);
+          GST_DEBUG_OBJECT (avi, "seeking to %" G_GINT64_FORMAT,
+              desired_offset);
 
           flags = GST_RIFF_IF_KEYFRAME;
           switch (GST_EVENT_SEEK_FORMAT (event)) {
@@ -559,9 +561,9 @@ gst_avi_demux_handle_src_event (GstPad * pad, GstEvent * event)
             avi->seek_flush =
                 (GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH);
             avi->seek_entry = entry->index_nr;
-            GST_DEBUG ("Will seek to entry %d", avi->seek_entry);
+            GST_DEBUG_OBJECT (avi, "Will seek to entry %d", avi->seek_entry);
           } else {
-            GST_DEBUG ("no index entry found for format=%d value=%"
+            GST_DEBUG_OBJECT (avi, "no index entry found for format=%d value=%"
                 G_GINT64_FORMAT, GST_EVENT_SEEK_FORMAT (event), desired_offset);
             res = FALSE;
           }
@@ -578,7 +580,6 @@ gst_avi_demux_handle_src_event (GstPad * pad, GstEvent * event)
       break;
   }
 
-done:
   gst_event_unref (event);
 
   return res;
@@ -883,8 +884,10 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
   GstPadTemplate *templ = NULL;
   GstPad *pad;
   avi_stream_context *stream;
-  gint blockalign = 0, bitrate = 0;
+  gint blockalign = 0, bitrate = 0, width = 0, height = 0;
   guint64 *locations = NULL;
+  GstTagList *list = gst_tag_list_new ();
+  gboolean have_tag = FALSE;
   union
   {
     gst_riff_strf_vids *vids;
@@ -984,7 +987,6 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
     case GST_RIFF_FCC_vids:
     {
       char *codec_name = NULL;
-      GstTagList *list = gst_tag_list_new ();
       guint32 tag;
 
       padname = g_strdup_printf ("video_%02d", avi->num_v_streams);
@@ -995,12 +997,14 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
         tag = strh->fcc_handler;
       caps = gst_riff_create_video_caps_with_data (tag,
           strh, strf.vids, extradata, initdata, &codec_name);
-      gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
-          codec_name, NULL);
-      gst_element_found_tags (GST_ELEMENT (avi), list);
-      gst_tag_list_free (list);
-      if (codec_name)
+      if (codec_name) {
+        gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
+            codec_name, NULL);
+        have_tag = TRUE;
         g_free (codec_name);
+      }
+      width = strf.vids->width;
+      height = strf.vids->height;
       g_free (strf.vids);
       avi->num_v_streams++;
       break;
@@ -1008,19 +1012,18 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
     case GST_RIFF_FCC_auds:
     {
       char *codec_name = NULL;
-      GstTagList *list = gst_tag_list_new ();
 
       padname = g_strdup_printf ("audio_%02d", avi->num_a_streams);
       templ = gst_element_class_get_pad_template (klass, "audio_%02d");
       caps =
           gst_riff_create_audio_caps_with_data (strf.auds->format, strh,
           strf.auds, extradata, initdata, &codec_name);
-      gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_AUDIO_CODEC,
-          codec_name, NULL);
-      gst_element_found_tags (GST_ELEMENT (avi), list);
-      gst_tag_list_free (list);
-      if (codec_name)
+      if (codec_name) {
+        gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_AUDIO_CODEC,
+            codec_name, NULL);
+        have_tag = TRUE;
         g_free (codec_name);
+      }
       blockalign = strf.auds->blockalign;
       bitrate = strf.auds->av_bps;
       g_free (strf.auds);
@@ -1030,18 +1033,17 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
     case GST_RIFF_FCC_iavs:
     {
       char *codec_name = NULL;
-      GstTagList *list = gst_tag_list_new ();
 
       padname = g_strdup_printf ("video_%02d", avi->num_v_streams);
       templ = gst_element_class_get_pad_template (klass, "video_%02d");
       caps = gst_riff_create_iavs_caps (strh->fcc_handler, strh, strf.iavs,
           &codec_name);
-      gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
-          codec_name, NULL);
-      gst_element_found_tags (GST_ELEMENT (avi), list);
-      gst_tag_list_free (list);
-      if (codec_name)
+      if (codec_name) {
+        gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
+            codec_name, NULL);
+        have_tag = TRUE;
         g_free (codec_name);
+      }
       g_free (strf.iavs);
       avi->num_v_streams++;
       break;
@@ -1076,6 +1078,8 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
   stream->skip = 0;
   stream->blockalign = blockalign;
   stream->bitrate = bitrate;
+  stream->width = width;
+  stream->height = height;
   stream->indexes = locations;
   gst_pad_set_element_private (pad, stream);
   avi->num_streams++;
@@ -1089,6 +1093,15 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
   if (extradata)
     gst_buffer_unref (extradata);
 
+  if (have_tag) {
+    GstEvent *event = gst_event_new_tag (list);
+
+    gst_element_found_tags (GST_ELEMENT (avi), list);
+    gst_pad_push (pad, GST_DATA (event));
+  } else {
+    gst_tag_list_free (list);
+  }
+
   return TRUE;
 
 skip_stream:
@@ -1313,7 +1326,8 @@ gst_avi_demux_stream_index (GstAviDemux * avi,
     stream->total_bytes += target->size;
     stream->total_frames++;
 
-    GST_DEBUG ("Adding index entry %d (%d) for stream %d of size %u "
+    GST_DEBUG_OBJECT (avi,
+        "Adding index entry %d (%d) for stream %d of size %u "
         "at offset %" G_GUINT64_FORMAT " and time %" GST_TIME_FORMAT,
         target->index_nr, stream->total_frames - 1,
         target->stream_nr, target->size, target->offset,
@@ -1326,7 +1340,7 @@ gst_avi_demux_stream_index (GstAviDemux * avi,
     avi_stream_context *stream;
 
     stream = &avi->stream[i];
-    GST_DEBUG ("stream %u: %u frames, %" G_GINT64_FORMAT " bytes",
+    GST_DEBUG_OBJECT (avi, "stream %u: %u frames, %" G_GINT64_FORMAT " bytes",
         i, stream->total_frames, stream->total_bytes);
   }
 
@@ -1589,7 +1603,7 @@ gst_avi_demux_stream_scan (GstAviDemux * avi,
     stream->total_frames++;
 
     list = g_list_prepend (list, entry);
-    GST_DEBUG ("Added index entry %d (in stream: %d), offset %"
+    GST_DEBUG_OBJECT (avi, "Added index entry %d (in stream: %d), offset %"
         G_GUINT64_FORMAT ", time %" GST_TIME_FORMAT " for stream %d",
         index_size - 1, entry->frames_before, entry->offset,
         GST_TIME_ARGS (entry->ts), entry->stream_nr);
@@ -1935,7 +1949,7 @@ gst_avi_demux_stream_header (GstAviDemux * avi)
 
   /* at this point we know all the streams and we can signal the no more
    * pads signal */
-  GST_DEBUG ("signaling no more pads");
+  GST_DEBUG_OBJECT (avi, "signaling no more pads");
   gst_element_no_more_pads (GST_ELEMENT (avi));
 
   return TRUE;
@@ -1974,6 +1988,38 @@ gst_avi_demux_handle_seek (GstAviDemux * avi)
   return TRUE;
 }
 
+/*
+ * Invert DIB buffers... Takes existing buffer and
+ * returns either the buffer or a new one (with old
+ * one dereferenced).
+ */
+
+static inline void
+swap_line (guint8 * d1, guint8 * d2, guint8 * tmp, gint bytes)
+{
+  memcpy (tmp, d1, bytes);
+  memcpy (d1, d2, bytes);
+  memcpy (d2, tmp, bytes);
+}
+
+static GstBuffer *
+gst_avi_demux_invert (avi_stream_context * stream, GstBuffer * buf)
+{
+  gint y, h = stream->height, w = stream->width;
+  guint8 *tmp = g_malloc (w);
+
+  buf = gst_buffer_copy_on_write (buf);
+
+  for (y = 0; y < h / 2; y++) {
+    swap_line (GST_BUFFER_DATA (buf) + w * y,
+        GST_BUFFER_DATA (buf) + w * (h - 1 - y), tmp, w);
+  }
+
+  g_free (tmp);
+
+  return buf;
+}
+
 static gboolean
 gst_avi_demux_process_next_entry (GstAviDemux * avi)
 {
@@ -1996,7 +2042,8 @@ gst_avi_demux_process_next_entry (GstAviDemux * avi)
       avi_stream_context *stream;
 
       if (entry->stream_nr >= avi->num_streams) {
-        GST_DEBUG ("Entry has non-existing stream nr %d", entry->stream_nr);
+        GST_DEBUG_OBJECT (avi,
+            "Entry has non-existing stream nr %d", entry->stream_nr);
         continue;
       }
 
@@ -2025,6 +2072,9 @@ gst_avi_demux_process_next_entry (GstAviDemux * avi)
           GST_ERROR ("Failed to read %d bytes of data", entry->size);
           return FALSE;
         }
+        if (stream->strh->fcc_handler == GST_MAKE_FOURCC ('D', 'I', 'B', ' ')) {
+          buf = gst_avi_demux_invert (stream, buf);
+        }
         if (entry->flags & GST_RIFF_IF_KEYFRAME) {
           GST_BUFFER_FLAG_SET (buf, GST_BUFFER_KEY_UNIT);
         }
@@ -2037,7 +2087,8 @@ gst_avi_demux_process_next_entry (GstAviDemux * avi)
         gst_pad_push (stream->pad, GST_DATA (buf));
         processed = TRUE;
       } else {
-        GST_DEBUG ("Unusable pad or zero chunksize, skipping entry");
+        GST_DEBUG_OBJECT (avi,
+            "Unusable pad or zero chunksize, skipping entry");
         processed = TRUE;
       }
       stream->current_frame = entry->frames_before + 1;
@@ -2114,10 +2165,15 @@ gst_avi_demux_stream_data (GstAviDemux * avi)
       } else {
         GstClockTime dur_ts;
 
+        if (stream->strh->fcc_handler == GST_MAKE_FOURCC ('D', 'I', 'B', ' ')) {
+          buf = gst_avi_demux_invert (stream, buf);
+        }
+
         GST_BUFFER_TIMESTAMP (buf) = next_ts;
         gst_pad_query (stream->pad, GST_QUERY_POSITION, &format, &dur_ts);
         GST_BUFFER_DURATION (buf) = dur_ts - next_ts;
-        GST_DEBUG ("Pushing buffer with time=%" GST_TIME_FORMAT " over pad %s",
+        GST_DEBUG_OBJECT (avi,
+            "Pushing buffer with time=%" GST_TIME_FORMAT " over pad %s",
             GST_TIME_ARGS (next_ts), gst_pad_get_name (stream->pad));
         gst_pad_push (stream->pad, GST_DATA (buf));
       }
