@@ -52,11 +52,14 @@ typedef struct
   gint		 nstreams;
   GList		*streaminfo;
 
-  gint		 naudiopads;
-  gint		 nvideopads;
-  gint		 nunknownpads;
-
-  GList		*preroll_elems;
+  struct {
+    gint	 npads;
+    GstBin      *bin;
+    GstElement	*preroll;
+    GstElement	*selector;
+    gboolean	 done;
+#define NUM_TYPES 3
+  } type[NUM_TYPES]; /* AUDIO, VIDEO, TEXT */
 } GstPlayBaseGroup;
 
 struct _GstPlayBaseBin {
@@ -66,11 +69,14 @@ struct _GstPlayBaseBin {
   gboolean	 threaded;
   guint64	 queue_size;
 
+  gint		 current[NUM_TYPES];
+
   /* internal thread */
   GstElement	*thread;
-  gchar 	*uri;
+  gchar 	*uri, *suburi;
   GstElement	*source;
   GstElement	*decoder;
+  GstElement	*subtitle; /* additional filesrc ! subparse bin */
   gboolean	 need_rebuild;
 
   /* group management */
@@ -95,6 +101,7 @@ struct _GstPlayBaseBinClass {
    * 100: buf=full (overrun) - will flush head of cache (latency) */
   void (*buffering)		(GstPlayBaseBin *play_base_bin,
 				 gint            percentage);
+  void (*group_switch)		(GstPlayBaseBin *play_base_bin);
 
   /* action signals */
   gboolean (*link_stream)	(GstPlayBaseBin *play_base_bin, 
