@@ -852,7 +852,8 @@ qt_type_find (GstTypeFind * tf, gpointer unused)
         strncmp (&data[4], "pnot", 4) != 0 &&
         strncmp (&data[4], "PICT", 4) != 0 &&
         strncmp (&data[4], "ftyp", 4) != 0 &&
-        strncmp (&data[4], "free", 4) != 0) {
+        strncmp (&data[4], "free", 4) != 0 &&
+        strncmp (&data[4], "skip", 4) != 0) {
       tip = 0;
       break;
     }
@@ -894,11 +895,28 @@ aiff_type_find (GstTypeFind * tf, gpointer unused)
 
   if (data && memcmp (data, "FORM", 4) == 0) {
     data += 8;
-    if (memcmp (data, "AIFF", 4) == 0)
+    if (memcmp (data, "AIFF", 4) == 0 || memcmp (data, "AIFC", 4) == 0)
       gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, AIFF_CAPS);
   }
 }
 
+/*** audio/x-aiff *********************************************/
+
+static GstStaticCaps svx_caps = GST_STATIC_CAPS ("audio/x-svx");
+
+#define SVX_CAPS gst_static_caps_get(&svx_caps)
+static void
+svx_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+
+  if (data && memcmp (data, "FORM", 4) == 0) {
+    data += 8;
+    if (memcmp (data, "8SVX", 4) == 0 || memcmp (data, "16SV", 4) == 0)
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, SVX_CAPS);
+  }
+}
+
 /*** audio/x-shorten ****************************************/
 
 static GstStaticCaps shn_caps = GST_STATIC_CAPS ("audio/x-shorten");
@@ -948,10 +966,30 @@ m4a_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** application/x-3gp *********************************************/
+
+/*
+ * The Q is there because variables can't start with a number.
+ */
+
+static GstStaticCaps q3gp_caps = GST_STATIC_CAPS ("application/x-3gp");
+
+#define Q3GP_CAPS (gst_static_caps_get(&q3gp_caps))
+static void
+q3gp_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 4, 8);
+
+  if (data && memcmp (data, "ftyp3gp4", 8) == 0) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, Q3GP_CAPS);
+  }
+}
+
 /*** audio/x-mod *********************************************/
 
 static GstStaticCaps mod_caps = GST_STATIC_CAPS ("audio/x-mod");
 
+
 #define MOD_CAPS gst_static_caps_get(&mod_caps)
 /* FIXME: M15 CheckType to do */
 static void
@@ -1151,6 +1189,61 @@ tiff_type_find (GstTypeFind * tf, gpointer ununsed)
   }
 }
 
+static GstStaticCaps sds_caps = GST_STATIC_CAPS ("audio/x-sds");
+
+#define SDS_CAPS (gst_static_caps_get(&sds_caps))
+static void
+sds_type_find (GstTypeFind * tf, gpointer ununsed)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+  guint8 mask[4] = { 0xFF, 0xFF, 0x80, 0xFF };
+  guint8 match[4] = { 0xF0, 0x7E, 0, 0x01 };
+  gint x;
+
+  if (data) {
+    for (x = 0; x < 4; x++) {
+      if ((data[x] & mask[x]) != match[x]) {
+        return;
+      }
+    }
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, SDS_CAPS);
+  }
+}
+
+static GstStaticCaps ircam_caps = GST_STATIC_CAPS ("audio/x-ircam");
+
+#define IRCAM_CAPS (gst_static_caps_get(&ircam_caps))
+static void
+ircam_type_find (GstTypeFind * tf, gpointer ununsed)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+  guint8 mask[4] = { 0xFF, 0xFF, 0xF8, 0xFF };
+  guint8 match[4] = { 0x64, 0xA3, 0x00, 0x00 };
+  gint x;
+  gboolean matched = TRUE;
+
+  if (!data) {
+    return;
+  }
+  for (x = 0; x < 4; x++) {
+    if ((data[x] & mask[x]) != match[x]) {
+      matched = FALSE;
+    }
+  }
+  if (matched) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, IRCAM_CAPS);
+    return;
+  }
+  // now try the reverse version
+  matched = TRUE;
+  for (x = 0; x < 4; x++) {
+    if ((data[x] & mask[3 - x]) != match[3 - x]) {
+      matched = FALSE;
+    }
+  }
+}
+
+
 /*** video/x-matroska ********************/
 static GstStaticCaps matroska_caps = GST_STATIC_CAPS ("video/x-matroska");
 
@@ -1230,8 +1323,36 @@ dv_type_find (GstTypeFind * tf, gpointer private)
   }
 }
 
-/*** audio/x-vorbis ***********************************************************/
 
+/*** application/ogg and application/x-annodex *****************************/
+static GstStaticCaps ogg_caps = GST_STATIC_CAPS ("application/ogg");
+static GstStaticCaps annodex_caps = GST_STATIC_CAPS ("application/x-annodex");
+static GstStaticCaps ogganx_caps =
+    GST_STATIC_CAPS ("application/ogg; application/x-annodex");
+#define OGGANX_CAPS (gst_static_caps_get(&ogganx_caps))
+
+static void
+ogganx_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 28, 8);
+  gboolean is_annodex = FALSE;
+
+  if ((data != NULL) && (memcmp (data, "fishead\0", 8) == 0)) {
+    is_annodex = TRUE;
+  }
+
+  data = gst_type_find_peek (tf, 0, 4);
+  if ((data != NULL) && (memcmp (data, "OggS", 4) == 0)) {
+    if (is_annodex == TRUE) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM,
+          gst_static_caps_get (&annodex_caps));
+    }
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM,
+        gst_static_caps_get (&ogg_caps));
+  }
+}
+
+/*** audio/x-vorbis ***********************************************************/
 static GstStaticCaps vorbis_caps = GST_STATIC_CAPS ("audio/x-vorbis");
 
 #define VORBIS_CAPS (gst_static_caps_get(&vorbis_caps))
@@ -1328,6 +1449,21 @@ ogmaudio_type_find (GstTypeFind * tf, gpointer private)
   }
 }
 
+static GstStaticCaps ogmtext_caps = GST_STATIC_CAPS ("application/x-ogm-text");
+
+#define OGMTEXT_CAPS (gst_static_caps_get(&ogmtext_caps))
+static void
+ogmtext_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 9);
+
+  if (data) {
+    if (memcmp (data, "\001text\000\000\000\000", 9) != 0)
+      return;
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, OGMTEXT_CAPS);
+  }
+}
+
 /*** audio/x-speex ***********************************************************/
 
 static GstStaticCaps speex_caps = GST_STATIC_CAPS ("audio/x-speex");
@@ -1364,8 +1500,59 @@ speex_type_find (GstTypeFind * tf, gpointer private)
   }
 }
 
-/*** generic typefind for streams that have some data at a specific position***/
+/*** application/x-ogg-skeleton ***********************************************************/
+static GstStaticCaps ogg_skeleton_caps =
+GST_STATIC_CAPS ("application/x-ogg-skeleton");
+#define OGG_SKELETON_CAPS (gst_static_caps_get(&ogg_skeleton_caps))
+static void
+oggskel_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 12);
 
+  if (data) {
+    /* 8 byte string "fishead\0" for the ogg skeleton stream */
+    if (memcmp (data, "fishead\0", 8) != 0)
+      return;
+    data += 8;
+
+    /* Require that the header contains version 3.0 */
+    if (GST_READ_UINT16_LE (data) != 3)
+      return;
+    data += 2;
+    if (GST_READ_UINT16_LE (data) != 0)
+      return;
+
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, OGG_SKELETON_CAPS);
+  }
+}
+
+static GstStaticCaps cmml_caps = GST_STATIC_CAPS ("text/x-cmml");
+
+#define CMML_CAPS (gst_static_caps_get(&cmml_caps))
+static void
+cmml_type_find (GstTypeFind * tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 12);
+
+  if (data) {
+    /* 8 byte string "CMML\0\0\0\0" for the magic number */
+    if (memcmp (data, "CMML\0\0\0\0", 8) != 0)
+      return;
+    data += 8;
+
+    /* Require that the header contains at least version 2.0 */
+    if (GST_READ_UINT16_LE (data) < 2)
+      return;
+    data += 2;
+
+    if (GST_READ_UINT16_LE (data) != 0)
+      return;
+
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, CMML_CAPS);
+  }
+}
+
+/*** generic typefind for streams that have some data at a specific position***/
 typedef struct
 {
   guint8 *data;
@@ -1457,13 +1644,20 @@ plugin_init (GstPlugin * plugin)
   static gchar *musepack_exts[] = { "mpc", NULL };
   static gchar *mpeg_sys_exts[] = { "mpe", "mpeg", "mpg", NULL };
   static gchar *mpeg_video_exts[] = { "mpv", "mpeg", "mpg", NULL };
-  static gchar *ogg_exts[] = { "ogg", "ogm", NULL };
+  static gchar *ogg_exts[] = { "anx", "ogg", "ogm", NULL };
   static gchar *qt_exts[] = { "mov", NULL };
   static gchar *rm_exts[] = { "ra", "ram", "rm", "rmvb", NULL };
   static gchar *swf_exts[] = { "swf", "swfl", NULL };
   static gchar *utf8_exts[] = { "txt", NULL };
   static gchar *wav_exts[] = { "wav", NULL };
   static gchar *aiff_exts[] = { "aiff", "aif", "aifc", NULL };
+  static gchar *svx_exts[] = { "iff", "svx", NULL };
+  static gchar *paris_exts[] = { "paf", NULL };
+  static gchar *nist_exts[] = { "nist", NULL };
+  static gchar *voc_exts[] = { "voc", NULL };
+  static gchar *sds_exts[] = { "sds", NULL };
+  static gchar *ircam_exts[] = { "sf", NULL };
+  static gchar *w64_exts[] = { "w64", NULL };
   static gchar *shn_exts[] = { "shn", NULL };
   static gchar *ape_exts[] = { "ape", NULL };
   static gchar *uri_exts[] = { "ram", NULL };
@@ -1474,6 +1668,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *tiff_exts[] = { "tif", "tiff", NULL };
   static gchar *matroska_exts[] = { "mkv", "mka", NULL };
   static gchar *dv_exts[] = { "dv", "dif", NULL };
+  static gchar *amr_exts[] = { "amr", NULL };
   static gchar *sid_exts[] = { "sid", NULL };
   static gchar *xcf_exts[] = { "xcf", NULL };
   static gchar *mng_exts[] = { "mng", NULL };
@@ -1485,6 +1680,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *zip_exts[] = { "zip", NULL };
   static gchar *compress_exts[] = { "Z", NULL };
   static gchar *m4a_exts[] = { "m4a", NULL };
+  static gchar *q3gp_exts[] = { "3gp", NULL };
   static gchar *aac_exts[] = { "aac", NULL };
 
   GST_DEBUG_CATEGORY_INIT (type_find_debug, "typefindfunctions",
@@ -1500,6 +1696,8 @@ plugin_init (GstPlugin * plugin)
       musepack_exts, "MP+", 3, GST_TYPE_FIND_MAXIMUM - 1);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-au", GST_RANK_MARGINAL,
       au_exts, ".snd", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-au", GST_RANK_MARGINAL,
+      au_exts, "dns.", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-msvideo", GST_RANK_PRIMARY,
       avi_exts, "AVI ");
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-cdxa", GST_RANK_PRIMARY,
@@ -1527,8 +1725,8 @@ plugin_init (GstPlugin * plugin)
       mpeg1_sys_type_find, mpeg_sys_exts, MPEG_SYS_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg2", GST_RANK_SECONDARY,
       mpeg2_sys_type_find, mpeg_sys_exts, MPEG_SYS_CAPS, NULL);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "application/ogg", GST_RANK_PRIMARY,
-      ogg_exts, "OggS", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER (plugin, "application/ogg", GST_RANK_PRIMARY,
+      ogganx_type_find, ogg_exts, OGGANX_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg", GST_RANK_SECONDARY,
       mpeg_video_type_find, mpeg_video_exts, MPEG_VIDEO_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg-stream", GST_RANK_MARGINAL,
@@ -1547,6 +1745,22 @@ plugin_init (GstPlugin * plugin)
       "WAVE");
   TYPE_FIND_REGISTER (plugin, "audio/x-aiff", GST_RANK_SECONDARY,
       aiff_type_find, aiff_exts, AIFF_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "audio/x-svx", GST_RANK_SECONDARY,
+      svx_type_find, svx_exts, SVX_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-paris",
+      GST_RANK_SECONDARY, paris_exts, " paf", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-paris",
+      GST_RANK_SECONDARY, paris_exts, "fap ", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-nist",
+      GST_RANK_SECONDARY, nist_exts, "NIST", 4, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-voc",
+      GST_RANK_SECONDARY, voc_exts, "Creative", 8, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER (plugin, "audio/x-sds", GST_RANK_SECONDARY,
+      sds_type_find, sds_exts, SDS_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "audio/x-ircam", GST_RANK_SECONDARY,
+      ircam_type_find, ircam_exts, IRCAM_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-w64",
+      GST_RANK_SECONDARY, w64_exts, "riff", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "audio/x-shorten", GST_RANK_SECONDARY,
       shn_type_find, shn_exts, SHN_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-ape", GST_RANK_SECONDARY,
@@ -1565,6 +1779,10 @@ plugin_init (GstPlugin * plugin)
       matroska_type_find, matroska_exts, MATROSKA_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/x-dv", GST_RANK_SECONDARY, dv_type_find,
       dv_exts, DV_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-amr-nb-sh", GST_RANK_PRIMARY,
+      amr_exts, "#!AMR", 5, GST_TYPE_FIND_LIKELY);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-amr-wb-sh", GST_RANK_PRIMARY,
+      amr_exts, "#!AMR-WB", 7, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-sid", GST_RANK_MARGINAL,
       sid_exts, "PSID", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "image/x-xcf", GST_RANK_SECONDARY,
@@ -1594,10 +1812,18 @@ plugin_init (GstPlugin * plugin)
       ogmvideo_type_find, NULL, OGMVIDEO_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-ogm-audio", GST_RANK_PRIMARY,
       ogmaudio_type_find, NULL, OGMAUDIO_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-ogm-text", GST_RANK_PRIMARY,
+      ogmtext_type_find, NULL, OGMTEXT_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-speex", GST_RANK_PRIMARY,
       speex_type_find, NULL, SPEEX_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-ogg-skeleton", GST_RANK_PRIMARY,
+      oggskel_type_find, NULL, OGG_SKELETON_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "text/x-cmml", GST_RANK_PRIMARY,
+      cmml_type_find, NULL, CMML_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-m4a", GST_RANK_PRIMARY, m4a_type_find,
       m4a_exts, M4A_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-3gp", GST_RANK_PRIMARY,
+      q3gp_type_find, q3gp_exts, Q3GP_CAPS, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "application/x-executable",
       GST_RANK_MARGINAL, NULL, "\177ELF", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "adts_mpeg_stream", GST_RANK_SECONDARY,
