@@ -44,8 +44,8 @@ enum
 enum
 {
   ARG_0,
-  ARG_METHOD,
-  /* FILL ME */
+  ARG_METHOD
+      /* FILL ME */
 };
 
 static void gst_videoflip_base_init (gpointer g_class);
@@ -82,6 +82,7 @@ gst_videoflip_method_get_type (void)
         "Flip across upper right/lower left diagonal"},
     {0, NULL, NULL},
   };
+
   if (!videoflip_method_type) {
     videoflip_method_type = g_enum_register_static ("GstVideoflipMethod",
         videoflip_methods);
@@ -131,6 +132,7 @@ gst_videoflip_get_type (void)
       0,
       (GInstanceInitFunc) gst_videoflip_init,
     };
+
     videoflip_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstVideoflip",
         &videoflip_info, 0);
