@@ -453,6 +453,7 @@ gst_videotestsrc_handle_src_event (GstPad * pad, GstEvent * event)
 
   videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
   new_n_frames = videotestsrc->n_frames;
+  new_running_time = videotestsrc->running_time;
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
@@ -515,6 +516,7 @@ gst_videotestsrc_handle_src_event (GstPad * pad, GstEvent * event)
 
   if (videotestsrc->n_frames != new_n_frames) {
     videotestsrc->n_frames = new_n_frames;
+    videotestsrc->running_time = new_running_time;
     videotestsrc->need_discont = TRUE;
   }
 
