@@ -291,7 +291,7 @@ gst_v4lsrc_grab_frame (GstV4lSrc * v4lsrc, gint * num)
     }
     if (v4lsrc->quit) {
       g_mutex_unlock (v4lsrc->mutex_queue_state);
-      return TRUE;              /* it won't get through anyway */
+      return FALSE;
     }
     if (!gst_v4lsrc_queue_frame (v4lsrc, v4lsrc->queue_frame)) {
       g_mutex_unlock (v4lsrc->mutex_queue_state);
@@ -511,3 +511,57 @@ gst_v4lsrc_try_capture (GstV4lSrc * v4lsrc, gint width, gint height,
   /* if we got here, it worked! woohoo, the format is supported! */
   return TRUE;
 }
+
+const char *
+gst_v4lsrc_palette_name (int i)
+{
+  return v4l_palette_name[i];
+}
+
+gfloat
+gst_v4lsrc_get_fps (GstV4lSrc * v4lsrc)
+{
+  gint norm;
+  gint fps_index;
+  gfloat fps;
+  struct video_window *vwin = &GST_V4LELEMENT (v4lsrc)->vwin;
+
+  /* check if we have vwin window properties giving a framerate,
+   * as is done for webcams
+   * See http://www.smcc.demon.nl/webcam/api.html
+   * which is used for the Philips and qce-ga drivers */
+  fps_index = (vwin->flags >> 16) & 0x3F;       /* 6 bit index for framerate */
+
+  /* webcams have a non-zero fps_index */
+  if (fps_index != 0) {
+    gfloat current_fps;
+
+    /* index of 16 corresponds to 15 fps */
+    current_fps = fps_index * 15.0 / 16;
+    GST_LOG_OBJECT (v4lsrc, "device reports fps of %.4f", current_fps);
+    return current_fps;
+  }
+
+  if (!(v4lsrc->syncmode == GST_V4LSRC_SYNC_MODE_FIXED_FPS) &&
+      v4lsrc->clock != NULL && v4lsrc->handled > 0) {
+    /* try to get time from clock master and calculate fps */
+    GstClockTime time =
+        gst_clock_get_time (v4lsrc->clock) - v4lsrc->substract_time;
+    return v4lsrc->handled * GST_SECOND / time;
+  }
+
+  /* if that failed ... */
+
+  if (!GST_V4L_IS_OPEN (GST_V4LELEMENT (v4lsrc)))
+    return 0.;
+
+  if (!gst_v4l_get_chan_norm (GST_V4LELEMENT (v4lsrc), NULL, &norm))
+    return 0.;
+
+  if (norm == VIDEO_MODE_NTSC)
+    fps = 30000 / 1001;
+  else
+    fps = 25.;
+
+  return fps;
+}
