@@ -48,6 +48,7 @@ MotifWmHints, MwmHints;
 static void gst_ximagesink_buffer_free (GstBuffer * buffer);
 static void gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
     GstXImage * ximage);
+static void gst_ximagesink_send_pending_navigation (GstXImageSink * ximagesink);
 
 /* ElementFactory information */
 static GstElementDetails gst_ximagesink_details =
@@ -389,6 +390,7 @@ static GstXWindow *
 gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
 {
   GstXWindow *xwindow = NULL;
+  XGCValues values;
 
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
 
@@ -408,7 +410,8 @@ gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
       StructureNotifyMask | PointerMotionMask | KeyPressMask |
       KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
 
-  xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win, 0, NULL);
+  xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win,
+      0, &values);
 
   XMapRaised (ximagesink->xcontext->disp, xwindow->win);
 
@@ -976,6 +979,8 @@ gst_ximagesink_sink_link (GstPad * pad, const GstCaps * caps)
   if (!ret)
     return GST_PAD_LINK_REFUSED;
 
+  g_mutex_lock (ximagesink->stream_lock);
+
   /* if the caps contain pixel-aspect-ratio, they have to match ours,
    * otherwise linking should fail */
   par = gst_structure_get_value (structure, "pixel-aspect-ratio");
@@ -1008,6 +1013,8 @@ gst_ximagesink_sink_link (GstPad * pad, const GstCaps * caps)
     ximagesink->ximage = NULL;
   }
 
+  g_mutex_unlock (ximagesink->stream_lock);
+
   gst_x_overlay_got_desired_size (GST_X_OVERLAY (ximagesink),
       GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
 
@@ -1031,11 +1038,11 @@ gst_ximagesink_change_state (GstElement * element)
       /* call XSynchronize with the current value of synchronous */
       GST_DEBUG_OBJECT (ximagesink, "XSynchronize called with %s",
           ximagesink->synchronous ? "TRUE" : "FALSE");
+      g_mutex_lock (ximagesink->x_lock);
       XSynchronize (ximagesink->xcontext->disp, ximagesink->synchronous);
+      g_mutex_unlock (ximagesink->x_lock);
       break;
     case GST_STATE_READY_TO_PAUSED:
-      if (ximagesink->xwindow)
-        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
       ximagesink->time = 0;
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
@@ -1043,12 +1050,20 @@ gst_ximagesink_change_state (GstElement * element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
+      g_mutex_lock (ximagesink->stream_lock);
+      if (ximagesink->xwindow)
+        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
       ximagesink->framerate = 0;
       ximagesink->sw_scaling_failed = FALSE;
       GST_VIDEOSINK_WIDTH (ximagesink) = 0;
       GST_VIDEOSINK_HEIGHT (ximagesink) = 0;
+      g_mutex_unlock (ximagesink->stream_lock);
       break;
     case GST_STATE_READY_TO_NULL:
+      /* We are cleaning our resources here, yes i know chain is not running
+         but the interface can be called to set a window from a different thread
+         and that would crash */
+      g_mutex_lock (ximagesink->stream_lock);
       if (ximagesink->ximage) {
         gst_ximagesink_ximage_destroy (ximagesink, ximagesink->ximage);
         ximagesink->ximage = NULL;
@@ -1066,6 +1081,7 @@ gst_ximagesink_change_state (GstElement * element)
         gst_ximagesink_xcontext_clear (ximagesink);
         ximagesink->xcontext = NULL;
       }
+      g_mutex_unlock (ximagesink->stream_lock);
       break;
   }
 
@@ -1103,6 +1119,8 @@ gst_ximagesink_chain (GstPad * pad, GstData * data)
     gst_element_wait (GST_ELEMENT (ximagesink), ximagesink->time);
   }
 
+  g_mutex_lock (ximagesink->stream_lock);
+
   /* If this buffer has been allocated using our buffer management we simply
      put the ximage which is in the PRIVATE pointer */
   if (GST_BUFFER_FREE_DATA_FUNC (buf) == gst_ximagesink_buffer_free) {
@@ -1139,6 +1157,9 @@ gst_ximagesink_chain (GstPad * pad, GstData * data)
   gst_buffer_unref (buf);
 
   gst_ximagesink_handle_xevents (ximagesink, pad);
+  gst_ximagesink_send_pending_navigation (ximagesink);
+
+  g_mutex_unlock (ximagesink->stream_lock);
 }
 
 /* Buffer management */
@@ -1238,37 +1259,78 @@ gst_ximagesink_interface_init (GstImplementsInterfaceClass * klass)
   klass->supported = gst_ximagesink_interface_supported;
 }
 
+/*
+ * This function is called with the stream-lock held
+ */
+static void
+gst_ximagesink_send_pending_navigation (GstXImageSink * ximagesink)
+{
+  GSList *cur;
+  GSList *pend_events;
+
+  g_mutex_lock (ximagesink->nav_lock);
+  pend_events = ximagesink->pend_nav_events;
+  ximagesink->pend_nav_events = NULL;
+  g_mutex_unlock (ximagesink->nav_lock);
+
+  cur = pend_events;
+  while (cur) {
+    GstEvent *event = cur->data;
+    GstStructure *structure;
+    double x, y;
+    gint x_offset, y_offset;
+
+    if (event) {
+      structure = event->event_data.structure.structure;
+
+      if (!GST_PAD_PEER (GST_VIDEOSINK_PAD (ximagesink))) {
+        gst_event_unref (event);
+        cur = g_slist_next (cur);
+        continue;
+      }
+
+      /* We are not converting the pointer coordinates as there's no hardware
+         scaling done here. The only possible scaling is done by videoscale and
+         videoscale will have to catch those events and tranform the coordinates
+         to match the applied scaling. So here we just add the offset if the image
+         is centered in the window.  */
+
+      x_offset = ximagesink->xwindow->width - GST_VIDEOSINK_WIDTH (ximagesink);
+      y_offset =
+          ximagesink->xwindow->height - GST_VIDEOSINK_HEIGHT (ximagesink);
+
+      if (gst_structure_get_double (structure, "pointer_x", &x)) {
+        x -= x_offset / 2;
+        gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
+      }
+      if (gst_structure_get_double (structure, "pointer_y", &y)) {
+        y -= y_offset / 2;
+        gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
+      }
+
+      gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD (ximagesink)),
+          event);
+    }
+    cur = g_slist_next (cur);
+  }
+
+  g_slist_free (pend_events);
+}
+
 static void
 gst_ximagesink_navigation_send_event (GstNavigation * navigation,
     GstStructure * structure)
 {
   GstXImageSink *ximagesink = GST_XIMAGESINK (navigation);
   GstEvent *event;
-  gint x_offset, y_offset;
-  double x, y;
 
   event = gst_event_new (GST_EVENT_NAVIGATION);
   event->event_data.structure.structure = structure;
 
-  /* We are not converting the pointer coordinates as there's no hardware
-     scaling done here. The only possible scaling is done by videoscale and
-     videoscale will have to catch those events and tranform the coordinates
-     to match the applied scaling. So here we just add the offset if the image
-     is centered in the window.  */
-
-  x_offset = ximagesink->xwindow->width - GST_VIDEOSINK_WIDTH (ximagesink);
-  y_offset = ximagesink->xwindow->height - GST_VIDEOSINK_HEIGHT (ximagesink);
-
-  if (gst_structure_get_double (structure, "pointer_x", &x)) {
-    x += x_offset;
-    gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
-  }
-  if (gst_structure_get_double (structure, "pointer_y", &y)) {
-    y += y_offset;
-    gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
-  }
-
-  gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD (ximagesink)), event);
+  g_mutex_lock (ximagesink->nav_lock);
+  ximagesink->pend_nav_events =
+      g_slist_prepend (ximagesink->pend_nav_events, event);
+  g_mutex_unlock (ximagesink->nav_lock);
 }
 
 static void
@@ -1299,6 +1361,11 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     return;
   }
 
+  /* We acquire the stream lock while setting this window in the element.
+     We are basically cleaning tons of stuff replacing the old window, putting
+     images while we do that would surely crash */
+  g_mutex_lock (ximagesink->stream_lock);
+
   /* Clear image pool as the images are unusable anyway */
   gst_ximagesink_imagepool_clear (ximagesink);
 
@@ -1382,6 +1449,8 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
 
   if (xwindow)
     ximagesink->xwindow = xwindow;
+
+  g_mutex_unlock (ximagesink->stream_lock);
 }
 
 static void
@@ -1402,6 +1471,10 @@ gst_ximagesink_expose (GstXOverlay * overlay)
   if (!ximagesink->xwindow)
     return;
 
+  /* We don't want chain to iterate while we do that. We might act on random
+     cur_image and different geometry */
+  g_mutex_lock (ximagesink->stream_lock);
+
   gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
 
   /* We don't act on internal window from outside that could cause some thread
@@ -1413,6 +1486,8 @@ gst_ximagesink_expose (GstXOverlay * overlay)
 
   if (ximagesink->cur_image)
     gst_ximagesink_ximage_put (ximagesink, ximagesink->cur_image);
+
+  g_mutex_unlock (ximagesink->stream_lock);
 }
 
 static void
@@ -1448,7 +1523,9 @@ gst_ximagesink_set_property (GObject * object, guint prop_id,
       if (ximagesink->xcontext) {
         GST_DEBUG_OBJECT (ximagesink, "XSynchronize called with %s",
             ximagesink->synchronous ? "TRUE" : "FALSE");
+        g_mutex_lock (ximagesink->x_lock);
         XSynchronize (ximagesink->xcontext->disp, ximagesink->synchronous);
+        g_mutex_unlock (ximagesink->x_lock);
       }
       break;
     case ARG_PIXEL_ASPECT_RATIO:
@@ -1516,10 +1593,27 @@ gst_ximagesink_finalize (GObject * object)
     g_mutex_free (ximagesink->x_lock);
     ximagesink->x_lock = NULL;
   }
+  if (ximagesink->stream_lock) {
+    g_mutex_free (ximagesink->stream_lock);
+    ximagesink->stream_lock = NULL;
+  }
   if (ximagesink->pool_lock) {
     g_mutex_free (ximagesink->pool_lock);
     ximagesink->pool_lock = NULL;
   }
+  if (ximagesink->nav_lock) {
+    g_mutex_free (ximagesink->nav_lock);
+    ximagesink->nav_lock = NULL;
+  }
+
+  while (ximagesink->pend_nav_events) {
+    GstEvent *event = ximagesink->pend_nav_events->data;
+
+    ximagesink->pend_nav_events =
+        g_slist_delete_link (ximagesink->pend_nav_events,
+        ximagesink->pend_nav_events);
+    gst_event_unref (event);
+  }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -1554,7 +1648,7 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
   ximagesink->framerate = 0;
 
   ximagesink->x_lock = g_mutex_new ();
-
+  ximagesink->stream_lock = g_mutex_new ();
 
   ximagesink->image_pool = NULL;
   ximagesink->pool_lock = g_mutex_new ();
@@ -1564,6 +1658,9 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
 
   ximagesink->par = NULL;
 
+  ximagesink->nav_lock = g_mutex_new ();
+  ximagesink->pend_nav_events = NULL;
+
   GST_FLAG_SET (ximagesink, GST_ELEMENT_THREAD_SUGGESTED);
   GST_FLAG_SET (ximagesink, GST_ELEMENT_EVENT_AWARE);
 }
